package io.reisub.openosrs.agility;

import com.google.inject.Provides;
import io.reisub.openosrs.agility.tasks.Alch;
import io.reisub.openosrs.agility.tasks.HandleObstacle;
import io.reisub.openosrs.agility.tasks.PickupMark;
import io.reisub.openosrs.util.CScript;
import io.reisub.openosrs.util.Util;
import io.reisub.openosrs.util.tasks.Eat;
import io.reisub.openosrs.util.tasks.Run;
import lombok.extern.slf4j.Slf4j;
import net.runelite.api.events.ChatMessage;
import net.runelite.api.events.GameTick;
import net.runelite.api.events.HitsplatApplied;
import net.runelite.api.events.StatChanged;
import net.runelite.client.config.ConfigManager;
import net.runelite.client.eventbus.Subscribe;
import net.runelite.client.plugins.PluginDependency;
import net.runelite.client.plugins.PluginDescriptor;
import net.runelite.client.plugins.iutils.iUtils;
import org.pf4j.Extension;

import javax.inject.Inject;

@Extension
@PluginDependency(Util.class)
@PluginDependency(iUtils.class)
@PluginDescriptor(
	name = "Chaos Agility",
	description = "Hippity hoppity, jumps on your property",
	enabledByDefault = false
)
@Slf4j
public class Agility extends CScript {
	@Inject
	private Config config;

	private Alch alchTask;
	private HandleObstacle handleObstacleTask;
	private PickupMark pickupMarkTask;

	@Provides
	Config provideConfig(ConfigManager configManager) {
		return configManager.getConfig(Config.class);
	}

	@Override
	protected void onStart() {
		super.onStart();

		Eat eatTask = injector.getInstance(Eat.class);
		eatTask.setInterval(14, 24);

		Run runTask = injector.getInstance(Run.class);
		runTask.setInterval(70, 95);

		handleObstacleTask = injector.getInstance(HandleObstacle.class);
		pickupMarkTask = injector.getInstance(PickupMark.class);

		if (config.highAlch()) {
			alchTask = injector.getInstance(Alch.class);
		}

		tasks.add(eatTask);
		tasks.add(runTask);
		tasks.add(pickupMarkTask);
		if (config.highAlch()) {
			tasks.add(alchTask);
		}
		tasks.add(handleObstacleTask);

		handleObstacleTask.setReady(true);
	}

	@Subscribe
	private void onHitsplatApplied(HitsplatApplied event) {
		if (handleObstacleTask != null) {
			handleObstacleTask.onHitsplatApplied(event);
		}

		if (pickupMarkTask != null) {
			pickupMarkTask.onHitsplatApplied(event);
		}
	}

	@Subscribe
	private void onStatChanged(StatChanged event) {
		if (handleObstacleTask != null) {
			handleObstacleTask.onStatChanged(event);
		}

		if (pickupMarkTask != null) {
			pickupMarkTask.onStatChanged(event);
		}

		if (alchTask != null) {
			alchTask.onStatChanged(event);
		}
	}

	@Subscribe
	private void onGameTick(GameTick event) {
		if (handleObstacleTask != null) {
			handleObstacleTask.onGameTick(event);
		}

		if (alchTask != null) {
			alchTask.onGameTick(event);
		}
	}

	@Subscribe
	private void onChatMessage(ChatMessage chatMessage) {
		if (handleObstacleTask != null) {
			handleObstacleTask.onChatMessage(chatMessage);
		}
	}
}