package io.reisub.openosrs.autobones;

import com.google.inject.Provides;
import io.reisub.openosrs.util.Util;
import lombok.extern.slf4j.Slf4j;
import net.runelite.api.GameState;
import net.runelite.api.events.ItemContainerChanged;
import net.runelite.client.config.ConfigManager;
import net.runelite.client.eventbus.Subscribe;
import net.runelite.client.events.ConfigChanged;
import net.runelite.client.plugins.Plugin;
import net.runelite.client.plugins.PluginDependency;
import net.runelite.client.plugins.PluginDescriptor;
import net.runelite.client.plugins.iutils.game.Game;
import net.runelite.client.plugins.iutils.iUtils;
import org.pf4j.Extension;

import javax.inject.Inject;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

@Extension
@PluginDependency(Util.class)
@PluginDependency(iUtils.class)
@PluginDescriptor(
	name = "Chaos Autobones",
	description = "Buries ashes and scatters bones, or something like that.",
	enabledByDefault = false
)
@Slf4j
public class Autobones extends Plugin {
	@SuppressWarnings("unused")
	@Inject
	private Game game;

	@SuppressWarnings("unused")
	@Inject
	private Config config;

	@SuppressWarnings("unused")
	@Provides
	Config provideConfig(ConfigManager configManager) {
		return configManager.getConfig(Config.class);
	}

	private int[] itemIds;

	private ScheduledExecutorService executor;

	@Override
	protected void startUp() {
		log.info("Starting Chaos Autobones");

		itemIds = parseItemIds();

		executor = Executors.newSingleThreadScheduledExecutor();
	}

	@Override
	protected void shutDown() {
		log.info("Stopping Chaos Autobones");

		itemIds =  null;

		executor.shutdownNow();
	}

	@SuppressWarnings("unused")
	@Subscribe
	private void onConfigChanged(ConfigChanged event) {
		if (event.getGroup().equals("chaosautobones")) {
			itemIds = parseItemIds();
		}
	}

	@SuppressWarnings("unused")
	@Subscribe
	private void onItemContainerChanged(ItemContainerChanged event) {
		if (game.client().getGameState() != GameState.LOGGED_IN) return;

		if (config.bones().getId() == -1 && config.ashes().getId() == -1) return;

		executor.schedule(() -> {
			game.inventory().withId(itemIds).first().interact(0);
		}, 0, TimeUnit.MILLISECONDS);
	}

	private int[] parseItemIds() {
		int[] ids;

		if (config.allBelow()) {
			Ashes[] allAshes = Ashes.allBelow(config.ashes());
			Bones[] allBones = Bones.allBelow(config.bones());

			ids = new int[allAshes.length + allBones.length];
			int i = 0;

			for (Ashes a : allAshes) {
				ids[i++] = a.getId();
			}

			for (Bones b : allBones) {
				ids[i++] = b.getId();
			}
		} else {
			ids = new int[2];

			ids[0] = config.ashes().getId();
			ids[1] = config.bones().getId();
		}

		return ids;
	}
}