package io.reisub.openosrs.autodropper;

import com.google.inject.Provides;
import io.reisub.openosrs.util.Util;
import io.reisub.openosrs.util.enums.Activity;
import lombok.extern.slf4j.Slf4j;
import net.runelite.api.GameState;
import net.runelite.api.events.ConfigButtonClicked;
import net.runelite.api.events.ItemContainerChanged;
import net.runelite.client.config.ConfigManager;
import net.runelite.client.config.Keybind;
import net.runelite.client.eventbus.Subscribe;
import net.runelite.client.events.ConfigChanged;
import net.runelite.client.input.KeyListener;
import net.runelite.client.input.KeyManager;
import net.runelite.client.plugins.Plugin;
import net.runelite.client.plugins.PluginDependency;
import net.runelite.client.plugins.PluginDescriptor;
import net.runelite.client.plugins.iutils.game.Game;
import net.runelite.client.plugins.iutils.iUtils;
import org.pf4j.Extension;

import javax.inject.Inject;
import java.awt.event.KeyEvent;
import java.util.Locale;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

@Extension
@PluginDependency(Util.class)
@PluginDependency(iUtils.class)
@PluginDescriptor(
	name = "Chaos Autodropper",
	description = "Automatically drops stuff",
	enabledByDefault = false
)
@Slf4j
public class Autodropper extends Plugin implements KeyListener {
	@SuppressWarnings("unused")
	@Inject
	private Game game;

	@SuppressWarnings("unused")
	@Inject
	private Config config;

	@SuppressWarnings("unused")
	@Inject
	private KeyManager keyManager;

	@SuppressWarnings("unused")
	@Provides
	Config provideConfig(ConfigManager configManager) {
		return configManager.getConfig(Config.class);
	}

	private ScheduledExecutorService executor;
	private String[] itemNames;
	private int[] itemIds;

	@Override
	protected void startUp() {
		log.info("Starting Chaos Autodropper");

		itemNames = parseNames();
		itemIds = parseIds();

		keyManager.registerKeyListener(this);
		executor = Executors.newSingleThreadScheduledExecutor();
	}

	@Override
	protected void shutDown() {
		log.info("Stopping Chaos Autodropper");

		keyManager.unregisterKeyListener(this);
		executor.shutdownNow();
	}

	@SuppressWarnings("unused")
	@Subscribe
	private void onItemContainerChanged(ItemContainerChanged event) {
		if (game.client().getGameState() != GameState.LOGGED_IN || config.dropMethod() == DropMethod.NONE) return;

		if (config.dropMethod() == DropMethod.ON_ADD) {
			drop();
		} else if (config.dropMethod() == DropMethod.ON_FULL_INVENTORY && game.inventory().full()) {
			drop();
		}
	}

	@Subscribe
	@SuppressWarnings("unused")
	private void onConfigChanged(ConfigChanged event) {
		String name = this.getName().replaceAll(" ", "").toLowerCase(Locale.ROOT);

		if (event.getGroup().equals(name) && event.getKey().startsWith("item")) {
			itemNames = parseNames();
			itemIds = parseIds();
		}
	}

	@SuppressWarnings("unused")
	@Subscribe
	private void onConfigButtonPressed(ConfigButtonClicked event) {
		String name = this.getName().replaceAll(" ", "").toLowerCase(Locale.ROOT);

		if (event.getGroup().equals(name) && event.getKey().equals("reloadButton")) {
			itemNames = parseNames();
			itemIds = parseIds();
		}
	}

	private String[] parseNames() {
		String[] itemNames = config.itemNames().split(";");
		String[] seedNames = config.seedNames().split(";");
		String[] names = new String[itemNames.length + seedNames.length];

		int i = 0;

		for (String name : itemNames) {
			names[i++] = name.trim();
		}

		for (String name : seedNames) {
			names[i++] = name.trim();
		}

		return names;
	}

	private int[] parseIds() {
		if (config.itemIds().equals("")) return new int[]{};

		String[] idsStr = config.itemIds().split(";");
		int[] ids = new int[idsStr.length];

		for (int i = 0; i < idsStr.length; i++) {
			ids[i] = Integer.parseInt(idsStr[i].trim());
		}

		return ids;
	}

	@Override
	public void keyTyped(KeyEvent e) {}

	@Override
	public void keyPressed(KeyEvent e) {
		if (config.dropEnableHotkey() && config.dropHotkey().matches(e)) {
			drop();
		}
	}

	@Override
	public void keyReleased(KeyEvent e) {}

	private void drop() {
		if (!config.dropWhileItemSelected() && game.client().isItemSelected() == 1) return;

		executor.schedule(() -> {
			game.inventory().withName(itemNames).drop();
			game.inventory().withId(itemIds).drop();
		}, 0, TimeUnit.MILLISECONDS);
	}
}