package io.reisub.openosrs.birdhouse;

import com.google.inject.Provides;
import io.reisub.openosrs.birdhouse.tasks.*;
import io.reisub.openosrs.util.CScript;
import io.reisub.openosrs.util.Util;
import io.reisub.openosrs.util.tasks.Run;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import net.runelite.api.events.GameTick;
import net.runelite.client.config.ConfigManager;
import net.runelite.client.eventbus.Subscribe;
import net.runelite.client.input.KeyListener;
import net.runelite.client.plugins.PluginDependency;
import net.runelite.client.plugins.PluginDescriptor;
import net.runelite.client.plugins.iutils.iUtils;
import net.runelite.client.plugins.iutils.scene.Position;
import org.pf4j.Extension;

import javax.inject.Inject;
import java.awt.event.KeyEvent;
import java.time.Duration;
import java.time.Instant;
import java.util.HashMap;
import java.util.Map;

@Extension
@PluginDependency(Util.class)
@PluginDependency(iUtils.class)
@PluginDescriptor(
	name = "Chaos Birdhouse",
	description = "Automatic bird massacre",
	enabledByDefault = false
)
@Slf4j
public class Birdhouse extends CScript implements KeyListener {
	@Inject
	private Config config;

	@Provides
	Config provideConfig(ConfigManager configManager) {
		return configManager.getConfig(Config.class);
	}

	@Getter
	private Map<BirdhouseSpace, Instant> birdhouseTimers;

	@Getter
	private final Position islandPosition = new Position(3769, 3898, 0);

	private final Position hillHousePosition = new Position(3764, 3869, 1);

	private boolean active;

	@Override
	protected void onStart() {
		super.onStart();
		active = true;

		birdhouseTimers = new HashMap<>();
		birdhouseTimers.put(BirdhouseSpace.MEADOW_NORTH, Instant.EPOCH);
		birdhouseTimers.put(BirdhouseSpace.MEADOW_SOUTH, Instant.EPOCH);
		birdhouseTimers.put(BirdhouseSpace.VALLEY_NORTH, Instant.EPOCH);
		birdhouseTimers.put(BirdhouseSpace.VALLEY_SOUTH, Instant.EPOCH);

		Run runTask = injector.getInstance(Run.class);
		runTask.setInterval(70, 95);

		tasks.add(runTask);
		addTask(AddSeeds.class);
		addTask(BankSpores.class);
		addTask(BuildBirdhouse.class);
		addTask(CraftBirdhouse.class);
		addTask(EmptyBirdhouse.class);
		addTask(GetTools.class);
		addTask(GoToBirdhouse.class);
		addTask(GoToIsland.class);
		addTask(GoToMushroomMeadow.class);
		addTask(GoToVerdantValley.class);
		addTask(PickupSeed.class);
		addTask(PlantSeaweed.class);
		addTask(NoteSeaweed.class);
		addTask(HarvestSeaweed.class);
		addTask(Deposit.class);
	}

	@Override
	protected void onStop() {
		super.onStop();
		active = false;
	}

	@SuppressWarnings("unused")
	@Subscribe
	private void onGameTick(GameTick event) {
//		if (isLoggedIn()
//				&& game.localPlayer().position().regionID() == 14908) {
//			if (!active && game.localPlayer().position().distanceTo(hillHousePosition) < 10 && game.inventory().withNamePart("logs").count() == 4) {
//				execute();
//			} else if (active && game.inventory().all().isEmpty() && !bank.isOpen() && game.localPlayer().position().distanceTo(islandPosition) < 10) {
//				execute();
//			}
//		}
	}

	public boolean hasRecentlyBeenEmptied(BirdhouseSpace space) {
		return Duration.between(getBirdhouseTimers().get(space), Instant.now()).getSeconds() < 2000;
	}

	@Override
	public void keyTyped(KeyEvent e) {

	}

	@Override
	public void keyPressed(KeyEvent e) {
		if (config.birdhouseHotkey().matches(e)) {
			execute();
		}
	}

	@Override
	public void keyReleased(KeyEvent e) {

	}
}