package io.reisub.openosrs.blackjack;

import com.google.inject.Provides;
import io.reisub.openosrs.blackjack.tasks.Blackjack;
import io.reisub.openosrs.blackjack.tasks.Escape;
import io.reisub.openosrs.blackjack.tasks.Pickpocket;
import io.reisub.openosrs.util.Calculations;
import io.reisub.openosrs.util.Task;
import io.reisub.openosrs.util.Util;
import io.reisub.openosrs.util.tasks.Eat;
import io.reisub.openosrs.util.tasks.KittenTask;
import io.reisub.openosrs.util.tasks.Run;
import lombok.extern.slf4j.Slf4j;
import net.runelite.api.events.ChatMessage;
import net.runelite.api.events.ConfigButtonClicked;
import net.runelite.api.events.GameTick;
import net.runelite.client.config.ConfigManager;
import net.runelite.client.eventbus.Subscribe;
import net.runelite.client.plugins.PluginDependency;
import net.runelite.client.plugins.PluginDescriptor;
import net.runelite.client.plugins.iutils.iUtils;
import net.runelite.client.plugins.iutils.scripts.iScript;
import org.pf4j.Extension;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.List;

@Extension
@PluginDependency(Util.class)
@PluginDependency(iUtils.class)
@PluginDescriptor(
	name = "Chaos Blackjack",
	description = "",
	enabledByDefault = false
)
@Slf4j
public class BlackjackPlugin extends iScript {
	private final String BLACKJACK_SUCCESS = "You smack the bandit over the head and render them unconscious.";
	private final String BLACKJACK_FAIL = "Your blow only glances off the bandit's head.";
	private final String PICKPOCKET = "You pick the bandit's pocket.";

	private List<Task> tasks;
	private KittenTask kittenTask;
	private Blackjack blackjackTask;
	private Pickpocket pickpocketTask;
	private Escape escapeTask;
	private long lastAction = 0;

	@Provides
	BlackjackConfig provideConfig(ConfigManager configManager) {
		return configManager.getConfig(BlackjackConfig.class);
	}

	@Inject
	private Calculations calc;

	@Override
	protected void loop() {
		for (Task t : tasks) {
			if (t.validate()) {
				lastAction = System.currentTimeMillis();
				log.info(t.getStatus());
				t.execute();
				break;
			}
		}
	}

	@Override
	protected void onStart() {
		log.info("Starting Chaos Blackjack");

		Eat eatTask = injector.getInstance(Eat.class);
		eatTask.setWait(false);
		eatTask.setInterval(14, 24);

		Run runTask = injector.getInstance(Run.class);
		runTask.setInterval(70, 95);

		kittenTask = KittenTask.getInstance(injector);
		pickpocketTask = injector.getInstance(Pickpocket.class);
		blackjackTask = injector.getInstance(Blackjack.class);
		escapeTask = injector.getInstance(Escape.class);

		tasks = new ArrayList<>();
		tasks.add(kittenTask);
		tasks.add(escapeTask);
		tasks.add(pickpocketTask);
		tasks.add(blackjackTask);
		tasks.add(eatTask);
		tasks.add(runTask);

		blackjackTask.setReady(true);
	}

	@Override
	protected void onStop() {
		log.info("Stopping Chaos Blackjack");
		if (tasks != null) {
			tasks.clear();
		}

		KittenTask.handleKitten = false;
	}

	@Subscribe
	private void onConfigButtonPressed(ConfigButtonClicked configButtonClicked) {
		if (configButtonClicked.getKey().equals("startButton")) {
			execute();
		}
	}

	@Subscribe
	private void onChatMessage(ChatMessage chatMessage) {
		if (kittenTask != null) {
			kittenTask.onChatMessage(chatMessage);
		}

		if (pickpocketTask != null && blackjackTask != null) {
			String msg = chatMessage.getMessage();
			if (msg.contains(BLACKJACK_SUCCESS) || msg.contains(BLACKJACK_FAIL)) {
				blackjackTask.setNextKnockoutTick(game.client().getTickCount() + calc.random(3, 5));
				pickpocketTask.setReady(true);
			}

			if (msg.contains(BLACKJACK_FAIL)) {
				blackjackTask.setReady(true);
			}

			if (msg.contains(PICKPOCKET)) {
				if (pickpocketTask.getSuccessivePickpockets() < 2) {
					pickpocketTask.setReady(true);
				} else {
					pickpocketTask.setSuccessivePickpockets(0);
					blackjackTask.setReady(true);
				}
			}
		}
	}

	@Subscribe
	private void onGameTick(GameTick event) {
		if (System.currentTimeMillis() > lastAction + calc.random(5000, 6000)) {
			log.info("idle, blackjacking");
			pickpocketTask.setSuccessivePickpockets(0);
			blackjackTask.setReady(true);
		}
	}
}