/*
 * Copyright (c) 2018, Andrew EP | ElPinche256 <https://github.com/ElPinche256>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package io.reisub.openosrs.consume;

import net.runelite.client.config.ConfigGroup;
import net.runelite.client.config.ConfigItem;
import net.runelite.client.config.ConfigSection;

@ConfigGroup("chaosconsume")

public interface Config extends net.runelite.client.config.Config {
    @ConfigSection(
            keyName = "eatConfig",
            name = "Eating Configuration",
            description = "Configure when to eat",
            position = 1
    )
    String eatConfig = "eatConfig";

    @ConfigItem(
            keyName = "enableEating",
            name = "Enable eating",
            description = "Enable automatic eating based on the minimum and maximum values set below.",
            section = "eatConfig",
            position = 2
    )
    default boolean enableEating() {
        return true;
    }

    @ConfigItem(
            keyName = "minEatHP",
            name = "Minimum Eat HP",
            description = "Minimum HP to eat at.",
            section = "eatConfig",
            position = 3
    )
    default int minEatHP() {
        return 10;
    }

    @ConfigItem(
            keyName = "maxEatHP",
            name = "Maximum Eat HP",
            description = "Highest HP to potentially eat at.",
            section = "eatConfig",
            position = 4
    )
    default int maxEatHP() {
        return 20;
    }

    @ConfigItem(
            keyName = "comboEat",
            name = "Combo eat",
            description = "Combo karambwan with pot and/or other food.",
            section = "eatConfig",
            position = 6
    )
    default boolean comboEat() {
        return true;
    }

    @ConfigItem(
            keyName = "eatWarnings",
            name = "Warnings",
            description = "Enable warning messages in chat if you don't have any food.",
            section = "eatConfig",
            hidden = true,
            unhide = "enableEating",
            position = 7
    )
    default boolean eatWarnings() {
        return true;
    }

    @ConfigSection(
            keyName = "potsConfig",
            name = "Potions Configuration",
            description = "Configure when to pot",
            position = 10
    )
    String potsConfig = "potsConfig";

    @ConfigItem(
            keyName = "drinkAntiPoison",
            name = "Drink anti-poison",
            description = "Drink an anti-poison potion when poisoned.",
            section = "potsConfig",
            position = 11
    )
    default boolean drinkAntiPoison() {
        return true;
    }

    @ConfigItem(
            keyName = "antiPoisonWarnings",
            name = "Warnings",
            description = "Enable warning messages in chat if you don't have any anti-poison.",
            section = "potsConfig",
            hidden = true,
            unhide = "drinkAntiPoison",
            position = 12
    )
    default boolean antiPoisonWarnings() {
        return true;
    }

    @ConfigItem(
            keyName = "drinkAntiFire",
            name = "Drink anti-fire",
            description = "Drink an anti-fire potion when not under its effect or its effect is running out.",
            section = "potsConfig",
            position = 13
    )
    default boolean drinkAntiFire() {
        return false;
    }

    @ConfigItem(
            keyName = "antiFireWarnings",
            name = "Warnings",
            description = "Enable warning messages in chat if you don't have any anti-fire potions.",
            section = "potsConfig",
            hidden = true,
            unhide = "drinkAntiFire",
            position = 14
    )
    default boolean antiFireWarnings() {
        return true;
    }

    @ConfigItem(
            keyName = "drinkPrayer",
            name = "Drink prayer potions",
            description = "Drink a prayer potion when prayer points fall below a threshold.",
            section = "potsConfig",
            position = 15
    )
    default boolean drinkPrayer() {
        return false;
    }

    @ConfigItem(
            keyName = "minPrayerPoints",
            name = "Minimum prayer points",
            description = "Minimum prayer points to drink at.",
            section = "potsConfig",
            hidden = true,
            unhide = "drinkPrayer",
            position = 16
    )
    default int minPrayerPoints() {
        return 10;
    }

    @ConfigItem(
            keyName = "maxPrayerPoints",
            name = "Maximum prayer points",
            description = "Highest prayer points to potentially drink at",
            section = "potsConfig",
            hidden = true,
            unhide = "drinkPrayer",
            position = 17
    )
    default int maxPrayerPoints() {
        return 20;
    }

    @ConfigItem(
            keyName = "prayerWarnings",
            name = "Warnings",
            description = "Enable warning messages in chat if you don't have any prayer potions.",
            section = "potsConfig",
            hidden = true,
            unhide = "drinkPrayer",
            position = 18
    )
    default boolean prayerWarnings() {
        return true;
    }

    @ConfigItem(
            keyName = "drinkStrength",
            name = "Drink strength pots",
            description = "Enable to drink pots to boost strength.",
            section = "potsConfig",
            position = 19
    )
    default boolean drinkStrength() {
        return false;
    }

    @ConfigItem(
            keyName = "strengthLevel",
            name = "Strength level",
            description = "Drink strength boosting pot below this level.",
            section = "potsConfig",
            position = 20,
            hidden = true,
            unhide = "drinkStrength"
    )
    default int strengthLevel() {
        return 100;
    }

    @ConfigItem(
            keyName = "strengthWarnings",
            name = "Warnings",
            description = "Enable warning messages in chat if you don't have any strength potions.",
            section = "potsConfig",
            hidden = true,
            unhide = "drinkStrength",
            position = 21
    )
    default boolean strengthWarnings() {
        return true;
    }

    @ConfigItem(
            keyName = "drinkAttack",
            name = "Drink attack pots",
            description = "Enable to drink pots to boost attack.",
            section = "potsConfig",
            position = 22
    )
    default boolean drinkAttack() {
        return false;
    }

    @ConfigItem(
            keyName = "attackLevel",
            name = "Attack level",
            description = "Drink attack boosting pot below this level.",
            section = "potsConfig",
            position = 23,
            hidden = true,
            unhide = "drinkAttack"
    )
    default int attackLevel() {
        return 100;
    }

    @ConfigItem(
            keyName = "attackWarnings",
            name = "Warnings",
            description = "Enable warning messages in chat if you don't have any attack potions.",
            section = "potsConfig",
            hidden = true,
            unhide = "drinkAttack",
            position = 24
    )
    default boolean attackWarnings() {
        return true;
    }

    @ConfigItem(
            keyName = "drinkDefence",
            name = "Drink defence pots",
            description = "Enable to drink pots to boost defence.",
            section = "potsConfig",
            position = 25
    )
    default boolean drinkDefence() {
        return false;
    }

    @ConfigItem(
            keyName = "defenceLevel",
            name = "Defence level",
            description = "Drink defence boosting pot below this level.",
            section = "potsConfig",
            position = 26,
            hidden = true,
            unhide = "drinkDefence"
    )
    default int defenceLevel() {
        return 100;
    }

    @ConfigItem(
            keyName = "defenceWarnings",
            name = "Warnings",
            description = "Enable warning messages in chat if you don't have any defence potions.",
            section = "potsConfig",
            hidden = true,
            unhide = "drinkDefence",
            position = 27
    )
    default boolean defenceWarnings() {
        return true;
    }

    @ConfigItem(
            keyName = "drinkRanged",
            name = "Drink ranged pots",
            description = "Enable to drink pots to boost ranged.",
            section = "potsConfig",
            position = 28
    )
    default boolean drinkRanged() {
        return false;
    }

    @ConfigItem(
            keyName = "rangedLevel",
            name = "Ranged level",
            description = "Drink ranged boosting pot below this level.",
            section = "potsConfig",
            position = 29,
            hidden = true,
            unhide = "drinkRanged"
    )
    default int rangedLevel() {
        return 100;
    }

    @ConfigItem(
            keyName = "rangedWarnings",
            name = "Warnings",
            description = "Enable warning messages in chat if you don't have any ranged potions.",
            section = "potsConfig",
            hidden = true,
            unhide = "drinkRanged",
            position = 30
    )
    default boolean rangedWarnings() {
        return true;
    }

    @ConfigItem(
            keyName = "drinkMagic",
            name = "Drink magic pots",
            description = "Enable to drink pots to boost magic.",
            section = "potsConfig",
            position = 31
    )
    default boolean drinkMagic() {
        return false;
    }

    @ConfigItem(
            keyName = "magicLevel",
            name = "Magic level",
            description = "Drink magic boosting pot below this level.",
            section = "potsConfig",
            position = 32,
            hidden = true,
            unhide = "drinkMagic"
    )
    default int magicLevel() {
        return 100;
    }

    @ConfigItem(
            keyName = "magicWarnings",
            name = "Warnings",
            description = "Enable warning messages in chat if you don't have any magic potions.",
            section = "potsConfig",
            hidden = true,
            unhide = "drinkMagic",
            position = 33
    )
    default boolean magicWarnings() {
        return true;
    }

    @ConfigSection(
            keyName = "specialConfig",
            name = "Weapon Special Configuration",
            description = "Configure weapon specials",
            position = 40
    )
    String specialConfig = "specialConfig";

    @ConfigItem(
            keyName = "useSpecial",
            name = "Use special",
            description = "Use weapon special",
            section = "specialConfig",
            position = 41
    )
    default boolean useSpecial() { return false; }

    @ConfigItem(
            keyName = "specialWeapon",
            name = "Weapon",
            description = "What weapon to use the special of. Leaving this empty will use the currently equipped weapon.",
            section = "specialConfig",
            hidden = true,
            unhide = "useSpecial",
            position = 42
    )
    default String specialWeapon() { return ""; }

    @ConfigItem(
            keyName = "specialCost",
            name = "Special cost",
            description = "Cost of the special attack.",
            section = "specialConfig",
            hidden = true,
            unhide = "useSpecial",
            position = 43
    )
    default int specialCost() { return 100; }


}