package io.reisub.openosrs.consume;

import com.google.inject.Provides;
import io.reisub.openosrs.util.Calculations;
import io.reisub.openosrs.util.Util;
import lombok.extern.slf4j.Slf4j;
import net.runelite.api.*;
import net.runelite.api.events.*;
import net.runelite.api.widgets.Widget;
import net.runelite.api.widgets.WidgetInfo;
import net.runelite.client.config.ConfigManager;
import net.runelite.client.eventbus.Subscribe;
import net.runelite.client.events.ConfigChanged;
import net.runelite.client.plugins.Plugin;
import net.runelite.client.plugins.PluginDependency;
import net.runelite.client.plugins.PluginDescriptor;
import net.runelite.client.plugins.iutils.game.Game;
import net.runelite.client.plugins.iutils.game.InventoryItem;
import net.runelite.client.plugins.iutils.game.iWidget;
import net.runelite.client.plugins.iutils.iUtils;
import net.runelite.client.plugins.iutils.ui.Bank;
import org.pf4j.Extension;

import javax.inject.Inject;
import java.util.Set;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

@Extension
@PluginDependency(Util.class)
@PluginDependency(iUtils.class)
@PluginDescriptor(
	name = "Chaos Consume",
	description = "Eats, drinks, activates items/specials",
	enabledByDefault = false
)
@Slf4j
public class Consume extends Plugin {
	@SuppressWarnings("unused")
	@Inject
	private Game game;

	@SuppressWarnings("unused")
	@Inject
	private Config config;

	@SuppressWarnings("unused")
	@Inject
	private Calculations calc;

	@Inject
	private Bank bank;

	private final Set<Integer> IGNORE_FOOD = Set.of(ItemID.DWARVEN_ROCK_CAKE, ItemID.DWARVEN_ROCK_CAKE_7510);
	private final Set<Integer> DRINK_SET = Set.of(ItemID.JUG_OF_WINE, ItemID.SARADOMIN_BREW1, ItemID.SARADOMIN_BREW2, ItemID.SARADOMIN_BREW3, ItemID.SARADOMIN_BREW4, ItemID.XERICS_AID_1, ItemID.XERICS_AID_2, ItemID.XERICS_AID_3, ItemID.XERICS_AID_4, ItemID.XERICS_AID_1_20977, ItemID.XERICS_AID_2_20978, ItemID.XERICS_AID_3_20979, ItemID.XERICS_AID_4_20980, ItemID.XERICS_AID_1_20981, ItemID.XERICS_AID_2_20982, ItemID.XERICS_AID_3_20983, ItemID.XERICS_AID_4_20984, ItemID.BANDAGES);
	private final Set<Integer> ANTI_POISON_IDS = Set.of(ItemID.ANTIPOISON1, ItemID.ANTIPOISON2, ItemID.ANTIPOISON3, ItemID.ANTIPOISON4, ItemID.SUPERANTIPOISON1, ItemID.SUPERANTIPOISON2, ItemID.SUPERANTIPOISON3, ItemID.SUPERANTIPOISON4,
			ItemID.ANTIDOTE1, ItemID.ANTIDOTE2, ItemID.ANTIDOTE3, ItemID.ANTIDOTE4, ItemID.ANTIDOTE1_5958, ItemID.ANTIDOTE2_5956, ItemID.ANTIDOTE3_5954, ItemID.ANTIDOTE4_5952,
			ItemID.ANTIVENOM1, ItemID.ANTIVENOM2, ItemID.ANTIVENOM3, ItemID.ANTIVENOM4, ItemID.ANTIVENOM4_12913, ItemID.ANTIVENOM3_12915, ItemID.ANTIVENOM2_12917, ItemID.ANTIVENOM1_12919);
	private final Set<Integer> ANTI_FIRE_IDS = Set.of(ItemID.ANTIFIRE_POTION1, ItemID.ANTIFIRE_POTION2, ItemID.ANTIFIRE_POTION3, ItemID.ANTIFIRE_POTION4, ItemID.SUPER_ANTIFIRE_POTION1, ItemID.SUPER_ANTIFIRE_POTION2, ItemID.SUPER_ANTIFIRE_POTION3, ItemID.SUPER_ANTIFIRE_POTION4,
			ItemID.EXTENDED_ANTIFIRE1, ItemID.EXTENDED_ANTIFIRE2, ItemID.EXTENDED_ANTIFIRE3, ItemID.EXTENDED_ANTIFIRE4, ItemID.EXTENDED_SUPER_ANTIFIRE1, ItemID.EXTENDED_SUPER_ANTIFIRE2, ItemID.EXTENDED_SUPER_ANTIFIRE3, ItemID.EXTENDED_SUPER_ANTIFIRE4);
	private final Set<Integer> PRAYER_IDS = Set.of(ItemID.PRAYER_POTION1, ItemID.PRAYER_POTION2, ItemID.PRAYER_POTION3, ItemID.PRAYER_POTION4,
			ItemID.SUPER_RESTORE1, ItemID.SUPER_RESTORE2, ItemID.SUPER_RESTORE3, ItemID.SUPER_RESTORE4, ItemID.BLIGHTED_SUPER_RESTORE1,
			ItemID.BLIGHTED_SUPER_RESTORE2, ItemID.BLIGHTED_SUPER_RESTORE3, ItemID.BLIGHTED_SUPER_RESTORE4, ItemID.EGNIOL_POTION_1,
			ItemID.EGNIOL_POTION_2, ItemID.EGNIOL_POTION_3, ItemID.EGNIOL_POTION_4);
	private final Set<Integer> STRENGTH_IDS = Set.of(ItemID.STRENGTH_POTION1, ItemID.STRENGTH_POTION2, ItemID.STRENGTH_POTION3, ItemID.STRENGTH_POTION4,
			ItemID.SUPER_STRENGTH1, ItemID.SUPER_STRENGTH2, ItemID.SUPER_STRENGTH3, ItemID.SUPER_STRENGTH4,
			ItemID.DIVINE_SUPER_STRENGTH_POTION1, ItemID.DIVINE_SUPER_STRENGTH_POTION2, ItemID.DIVINE_SUPER_STRENGTH_POTION3, ItemID.DIVINE_SUPER_STRENGTH_POTION4,
			ItemID.DIVINE_SUPER_COMBAT_POTION1, ItemID.DIVINE_SUPER_COMBAT_POTION2, ItemID.DIVINE_SUPER_COMBAT_POTION3, ItemID.DIVINE_SUPER_COMBAT_POTION4,
			ItemID.COMBAT_POTION1, ItemID.COMBAT_POTION2, ItemID.COMBAT_POTION3, ItemID.COMBAT_POTION4,
			ItemID.SUPER_COMBAT_POTION1, ItemID.SUPER_COMBAT_POTION2, ItemID.SUPER_COMBAT_POTION3, ItemID.SUPER_COMBAT_POTION4);
	private final Set<Integer> ATTACK_IDS = Set.of(ItemID.ATTACK_POTION1, ItemID.ATTACK_POTION2, ItemID.ATTACK_POTION3, ItemID.ATTACK_POTION4,
			ItemID.SUPER_ATTACK1, ItemID.SUPER_ATTACK2, ItemID.SUPER_ATTACK3, ItemID.SUPER_ATTACK4,
			ItemID.DIVINE_SUPER_ATTACK_POTION1, ItemID.DIVINE_SUPER_ATTACK_POTION2, ItemID.DIVINE_SUPER_ATTACK_POTION3, ItemID.DIVINE_SUPER_ATTACK_POTION4,
			ItemID.DIVINE_SUPER_COMBAT_POTION1, ItemID.DIVINE_SUPER_COMBAT_POTION2, ItemID.DIVINE_SUPER_COMBAT_POTION3, ItemID.DIVINE_SUPER_COMBAT_POTION4,
			ItemID.COMBAT_POTION1, ItemID.COMBAT_POTION2, ItemID.COMBAT_POTION3, ItemID.COMBAT_POTION4,
			ItemID.SUPER_COMBAT_POTION1, ItemID.SUPER_COMBAT_POTION2, ItemID.SUPER_COMBAT_POTION3, ItemID.SUPER_COMBAT_POTION4);
	private final Set<Integer> DEFENCE_IDS = Set.of(ItemID.DEFENCE_POTION1, ItemID.DEFENCE_POTION2, ItemID.DEFENCE_POTION3, ItemID.DEFENCE_POTION4,
			ItemID.SUPER_DEFENCE1, ItemID.SUPER_DEFENCE2, ItemID.SUPER_DEFENCE3, ItemID.SUPER_DEFENCE4,
			ItemID.DIVINE_SUPER_DEFENCE_POTION1, ItemID.DIVINE_SUPER_DEFENCE_POTION2, ItemID.DIVINE_SUPER_DEFENCE_POTION3, ItemID.DIVINE_SUPER_DEFENCE_POTION4,
			ItemID.DIVINE_SUPER_COMBAT_POTION1, ItemID.DIVINE_SUPER_COMBAT_POTION2, ItemID.DIVINE_SUPER_COMBAT_POTION3, ItemID.DIVINE_SUPER_COMBAT_POTION4,
			ItemID.SUPER_COMBAT_POTION1, ItemID.SUPER_COMBAT_POTION2, ItemID.SUPER_COMBAT_POTION3, ItemID.SUPER_COMBAT_POTION4);
	private final Set<Integer> RANGED_IDS = Set.of(ItemID.RANGING_POTION1, ItemID.RANGING_POTION2, ItemID.RANGING_POTION3, ItemID.RANGING_POTION4,
			ItemID.BASTION_POTION1, ItemID.BASTION_POTION2, ItemID.BASTION_POTION3, ItemID.BASTION_POTION4,
			ItemID.DIVINE_RANGING_POTION1, ItemID.DIVINE_RANGING_POTION2, ItemID.DIVINE_RANGING_POTION3, ItemID.DIVINE_RANGING_POTION4,
			ItemID.DIVINE_BASTION_POTION1, ItemID.DIVINE_BASTION_POTION2, ItemID.DIVINE_BASTION_POTION3, ItemID.DIVINE_BASTION_POTION4,
			ItemID.SUPER_RANGING_1, ItemID.SUPER_RANGING_2, ItemID.SUPER_RANGING_3, ItemID.SUPER_RANGING_4);
	private final Set<Integer> MAGIC_IDS = Set.of(ItemID.MAGIC_POTION1, ItemID.MAGIC_POTION2, ItemID.MAGIC_POTION3, ItemID.MAGIC_POTION4,
			ItemID.BATTLEMAGE_POTION1, ItemID.BATTLEMAGE_POTION2, ItemID.BATTLEMAGE_POTION3, ItemID.BATTLEMAGE_POTION4,
			ItemID.DIVINE_MAGIC_POTION1, ItemID.DIVINE_MAGIC_POTION2, ItemID.DIVINE_MAGIC_POTION3, ItemID.DIVINE_MAGIC_POTION4,
			ItemID.DIVINE_BATTLEMAGE_POTION1, ItemID.DIVINE_BATTLEMAGE_POTION2, ItemID.DIVINE_BATTLEMAGE_POTION3, ItemID.DIVINE_BATTLEMAGE_POTION4);

	private ScheduledExecutorService executor;
	private long lastAte;
	private long lastPot;
	private int timeout;
	private int eatThreshold;
	private int prayerThreshold;
	private boolean shouldDrinkAntiPoison;
	private long lastAntiPoison;
	private boolean shouldDrinkAntiFire;
	private long lastAntiFire;
	private boolean shouldDrinkPrayer;
	private long lastPrayer;
	private boolean shouldDrinkStrength;
	private long lastStrength;
	private boolean shouldDrinkAttack;
	private long lastAttack;
	private boolean shouldDrinkDefence;
	private long lastDefence;
	private boolean shouldDrinkRanged;
	private long lastRanged;
	private boolean shouldDrinkMagic;
	private long lastMagic;
	private boolean shouldUseSpecial;

	@SuppressWarnings("unused")
	@Provides
    Config provideConfig(ConfigManager configManager) {
		return configManager.getConfig(Config.class);
	}

	@Override
	protected void startUp() {
		log.info("Starting Chaos Consume");

		generateNewEatThreshold();
		generateNewPrayerThreshold();

		executor = Executors.newSingleThreadScheduledExecutor();
	}

	@Override
	protected void shutDown() {
		log.info("Stopping Chaos Consume");

		executor.shutdownNow();
	}

	@SuppressWarnings("unused")
	@Subscribe
	private void onGameTick(GameTick event) {
		if (game.client() == null || game.localPlayer() == null || game.client().getGameState() != GameState.LOGGED_IN || bank.isOpen()) return;

		if (timeout > 0) {
			timeout--;
			return;
		}

		executor.schedule(this::tick, calc.random(50, 80), TimeUnit.MILLISECONDS);
	}

	@SuppressWarnings("unused")
	@Subscribe
	private void onVarbitChanged(VarbitChanged event) {
		if (game.client().getGameState() != GameState.LOGGED_IN) return;

		if (config.drinkAntiPoison()
				&& event.getIndex() == VarPlayer.POISON.getId()
				&& game.client().getVarpValue(VarPlayer.POISON.getId()) > 0) {
			shouldDrinkAntiPoison = true;
		}

		if (config.useSpecial()
				&& event.getIndex() == VarPlayer.SPECIAL_ATTACK_PERCENT.getId()
				&& game.client().getVarpValue(VarPlayer.SPECIAL_ATTACK_PERCENT.getId()) > config.specialCost()) {
			shouldUseSpecial = true;
		}
	}

	@SuppressWarnings("unused")
	@Subscribe
	private void onChatMessage(ChatMessage event) {
		if (game.client().getGameState() != GameState.LOGGED_IN) return;

		String BURN_MESSAGE = ("You're horribly burnt by the dragon fire!");
		String BURN_EXPIRE = ("antifire potion is about to expire.");

		String message = event.getMessage();

		if (config.drinkAntiFire() && (message.contains(BURN_MESSAGE) || message.contains(BURN_EXPIRE))) {
			shouldDrinkAntiFire = true;
		}
	}

	@SuppressWarnings("unused")
	@Subscribe
	private void onStatChanged(StatChanged event) {
		if (game.client().getGameState() != GameState.LOGGED_IN || timeout > 0) return;

		Skill skill = event.getSkill();
		int level = event.getBoostedLevel();

		checkSkill(skill, level);
	}

	@SuppressWarnings("unused")
	@Subscribe
	private void onGameStateChanged(GameStateChanged event) {
		if (event.getGameState() == GameState.LOGGED_IN) {
			timeout = 5;
		}
	}

	@SuppressWarnings("unused")
	@Subscribe
	private void onConfigChanged(ConfigChanged event) {
		if (!event.getGroup().equals("chaosconsume")) return;

		switch (event.getKey()) {
			case "minEatHP":
			case "maxEatHP":
				generateNewEatThreshold();
				break;
			case "minPrayerPoints":
			case "maxPrayerPoints":
				generateNewPrayerThreshold();
			case "drinkPrayer":
				checkSkill(Skill.PRAYER);
				break;
			case "strengthLevel":
				checkSkill(Skill.STRENGTH);
				break;
			case "attackLevel":
				checkSkill(Skill.ATTACK);
				break;
			case "defenceLevel":
				checkSkill(Skill.DEFENCE);
				break;
			case "rangedLevel":
				checkSkill(Skill.RANGED);
				break;
			case "magicLevel":
				checkSkill(Skill.MAGIC);
				break;
		}
	}

	private void tick() {

		int hp = game.modifiedLevel(Skill.HITPOINTS);

		if (config.enableEating() && hp <= eatThreshold && canEat()) {
			boolean[] ate = new boolean[1];

			game.inventory().withAction("Eat").withoutId(IGNORE_FOOD).findFirst().ifPresent((food) -> {
				food.interact(0);
				lastAte = game.ticks();
				ate[0] = true;
			});

			if (!ate[0]) {
				game.inventory().withId(DRINK_SET).findFirst().ifPresent((drink) -> {
					drink.interact(0);
					lastAte = game.ticks();
					ate[0] = true;
				});
			}

			if (!ate[0] && config.eatWarnings()) {
				game.utils.sendGameMessage("HP below threshold but you don't have any food!");
			} else {
				generateNewEatThreshold();
			}
		}

		if (shouldUseSpecial && game.client().getVarpValue(VarPlayer.SPECIAL_ATTACK_ENABLED.getId()) == 0) {
			shouldUseSpecial = false;

			if (game.client().getVar(Varbits.PVP_SPEC_ORB) == 0) {
				iWidget special = game.widget(WidgetInfo.MINIMAP_SPEC_CLICKBOX);
				if (special == null) return;

				special.interact(0);
			} else {

			}
		}

		if (shouldDrinkAntiPoison && canPot()) {
			if (!drinkPotion(ANTI_POISON_IDS) && config.antiPoisonWarnings()) {
				game.utils.sendGameMessage("Poisoned but you don't have anti-poison!");
			}

			shouldDrinkAntiPoison = false;
			lastAntiPoison = game.ticks();
		}

		if (shouldDrinkAntiFire && canPot()) {
			if (!drinkPotion(ANTI_FIRE_IDS) && config.antiFireWarnings()) {
				game.utils.sendGameMessage("Sustaining dragon fire damage but you don't have an anti-dragon fire potion!");
			}

			shouldDrinkAntiFire = false;
			lastAntiFire = game.ticks();
		}

		if (shouldDrinkPrayer && canPot()) {
			if (!drinkPotion(PRAYER_IDS) && config.prayerWarnings()) {
				game.utils.sendGameMessage("Prayer points below threshold but you don't have any prayer potions!");
			}

			shouldDrinkPrayer = false;
			lastPrayer = game.ticks();
		}

		if (shouldDrinkStrength && canPot()) {
			if (!drinkPotion(STRENGTH_IDS) && config.strengthWarnings()) {
				game.utils.sendGameMessage("Strength below threshold but you don't have any strength potions!");
			}

			shouldDrinkStrength = false;
			lastStrength = game.ticks();
		}

		if (shouldDrinkAttack && canPot()) {
			if (!drinkPotion(ATTACK_IDS) && config.attackWarnings()) {
				game.utils.sendGameMessage("Attack below threshold but you don't have any attack potions!");
			}

			shouldDrinkAttack = false;
			lastAttack = game.ticks();
		}

		if (shouldDrinkDefence && canPot()) {
			if (!drinkPotion(DEFENCE_IDS) && config.defenceWarnings()) {
				game.utils.sendGameMessage("Defence below threshold but you don't have any defence potions!");
			}

			shouldDrinkDefence = false;
			lastDefence = game.ticks();
		}

		if (shouldDrinkRanged && canPot()) {
			if (!drinkPotion(RANGED_IDS) && config.rangedWarnings()) {
				game.utils.sendGameMessage("Ranged below threshold but you don't have any ranged potions!");
			}

			shouldDrinkRanged = false;
			lastRanged = game.ticks();
		}

		if (shouldDrinkMagic && canPot()) {
			if (!drinkPotion(MAGIC_IDS) && config.magicWarnings()) {
				game.utils.sendGameMessage("Magic below threshold but you don't have any magic potions!");
			}

			shouldDrinkMagic = false;
			lastMagic = game.ticks();
		}

		if (config.enableEating() && hp < eatThreshold && config.comboEat()) {
			game.inventory().withId(ItemID.COOKED_KARAMBWAN, ItemID.COOKED_KARAMBWAN_3147, ItemID.COOKED_KARAMBWAN_23533).findFirst().ifPresent((food) -> {
				food.interact(0);
				lastAte = game.ticks();
			});
		}
	}

	private void generateNewEatThreshold() {
		eatThreshold = calc.random(config.minEatHP(), config.maxEatHP() + 1);
	}

	private void generateNewPrayerThreshold() {
		prayerThreshold = calc.random(config.minPrayerPoints(), config.maxPrayerPoints() + 1);
	}

	private boolean canPot() {
		return game.ticks() > lastPot + 3;
	}

	private boolean canEat() {
		return game.ticks() > lastAte + 3;
	}

	private void checkSkill(Skill skill) {
		checkSkill(skill, game.modifiedLevel(skill));
	}

	private void checkSkill(Skill skill, int level) {
		if (game.client().getGameState() != GameState.LOGGED_IN) return;

		switch (skill) {
			case PRAYER:
				if (config.drinkPrayer() && level <= prayerThreshold && game.ticks() > lastPrayer + 5) {
					shouldDrinkPrayer = true;
				}
				break;
			case STRENGTH:
				if (config.drinkStrength() && level <= config.strengthLevel() && game.ticks() > lastStrength + 5) {
					shouldDrinkStrength = true;
				}
				break;
			case ATTACK:
				if (config.drinkAttack() && level <= config.attackLevel() && game.ticks() > lastAttack + 5) {
					shouldDrinkAttack = true;
				}
				break;
			case DEFENCE:
				if (config.drinkDefence() && level <= config.defenceLevel() && game.ticks() > lastDefence + 5) {
					shouldDrinkDefence = true;
				}
				break;
			case RANGED:
				if (config.drinkRanged() && level <= config.rangedLevel() && game.ticks() > lastRanged + 5) {
					shouldDrinkRanged = true;
				}
				break;
			case MAGIC:
				if (config.drinkMagic() && level <= config.magicLevel() && game.ticks() > lastMagic + 5) {
					shouldDrinkMagic = true;
				}
				break;
		}
	}

	private boolean drinkPotion(Set<Integer> ids) {
		InventoryItem potion = game.inventory().withId(ids).first();
		if (potion == null) return false;

		potion.interact(0);
		lastPot = game.ticks();
		return true;
	}
}