package io.reisub.openosrs.cooker;

import com.google.inject.Provides;
import io.reisub.openosrs.cooker.tasks.Cook;
import io.reisub.openosrs.cooker.tasks.HandleBank;
import io.reisub.openosrs.util.CScript;
import io.reisub.openosrs.util.Util;
import io.reisub.openosrs.util.enums.Activity;
import io.reisub.openosrs.util.tasks.Run;
import lombok.extern.slf4j.Slf4j;
import net.runelite.api.AnimationID;
import net.runelite.api.GameState;
import net.runelite.api.events.AnimationChanged;
import net.runelite.api.events.ItemContainerChanged;
import net.runelite.client.config.ConfigManager;
import net.runelite.client.eventbus.Subscribe;
import net.runelite.client.plugins.PluginDependency;
import net.runelite.client.plugins.PluginDescriptor;
import net.runelite.client.plugins.iutils.iUtils;
import org.pf4j.Extension;

import javax.inject.Inject;

@Extension
@PluginDependency(Util.class)
@PluginDependency(iUtils.class)
@PluginDescriptor(
	name = "Chaos Cooker",
	description = "Cooks better than Gordon Ramsay",
	enabledByDefault = false
)
@Slf4j
public class Cooker extends CScript {
	@Inject
	private Config config;

	@Provides
    Config provideConfig(ConfigManager configManager) {
		return configManager.getConfig(Config.class);
	}

	@Override
	protected void onStart() {
		super.onStart();

		Run runTask = injector.getInstance(Run.class);
		runTask.setInterval(70, 95);

		tasks.add(runTask);
		addTask(HandleBank.class);
		addTask(Cook.class);
	}

	@SuppressWarnings("unused")
	@Subscribe
	private void onAnimationChanged(AnimationChanged event) {
		if (game.client().getGameState() != GameState.LOGGED_IN) return;
		if (event.getActor() != game.client().getLocalPlayer()) return;

		int animId = game.localPlayer().animation();
		switch (animId) {
			case AnimationID.COOKING_FIRE:
			case AnimationID.COOKING_RANGE:
				setActivity(Activity.COOKING);
		}
	}

		@SuppressWarnings("unused")
	@Subscribe
	private void onItemContainerChanged(ItemContainerChanged event) {
		if (!game.inventory().withId(config.rawFood()).exists()) {
			setActivity(Activity.IDLE);
		}
	}
}