/*
 * Copyright (c) 2018, Andrew EP | ElPinche256 <https://github.com/ElPinche256>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package io.reisub.openosrs.fighter;

import net.runelite.client.config.*;

@ConfigGroup("chaosfighter")

public interface Config extends net.runelite.client.config.Config {
    @ConfigSection(
        keyName = "targetConfiguration",
        name = "Target Configuration",
        description = "Configure target to fight",
        position = 0
    )
    String targetConfiguration = "targetConfiguration";

    @ConfigItem(
            keyName = "targetNames",
            name = "Target names",
            description = "The names of your targets, separated with a semicolon.",
            section = "targetConfiguration",
            position = 1
    )
    default String targetNames() {
        return "";
    }

    @ConfigItem(
            keyName = "targetIds",
            name = "Target IDs",
            description = "The IDs of your targets, separated with a semicolon.",
            section = "targetConfiguration",
            position = 2
    )
    default String targetIds() {
        return "";
    }

    @Range(
            min = 1,
            max = 64
    )
    @ConfigItem(
            keyName = "searchRadius",
            name = "Search radius",
            description = "Search radius in which to search the target NPC.",
            section = "targetConfiguration",
            position = 3
    )
    default int searchRadius() {
        return 20;
    }

    @ConfigItem(
            keyName = "usePathfinding",
            name = "Use pathfinding",
            description = "Use pathfinding to find nearest target. This is a lot more expensive CPU-wise but is recommended when fighting in an area with obstacles.",
            section = "targetConfiguration",
            position = 4
    )
    default boolean usePathfinding() {
        return false;
    }

    @Range(
            min = 1,
            max = 64
    )
    @ConfigItem(
            keyName = "pathfindingSearchRadius",
            name = "Pathfinding search radius",
            description = "Search radius in which to search the target NPC using pathfinding. Be careful, setting this too high will freeze the game. If no NPC is found within the pathfinding radius it will fall back to the non-pathfinding radius.",
            section = "targetConfiguration",
            hidden = true,
            unhide = "usePathfinding",
            position = 5
    )
    default int pathfindingSearchRadius() {
        return 8;
    }

    @ConfigItem(
            keyName = "startButton",
            name = "Start/Stop",
            description = "Start the script",
            position = 100
    )
    default Button startButton() {
        return new Button();
    }
}