package io.reisub.openosrs.fighter;

import com.google.inject.Provides;
import io.reisub.openosrs.util.CScript;
import io.reisub.openosrs.util.Util;
import io.reisub.openosrs.util.tasks.Run;
import lombok.Getter;
import lombok.Setter;
import lombok.extern.slf4j.Slf4j;
import net.runelite.api.NPC;
import net.runelite.api.events.ActorDeath;
import net.runelite.client.config.ConfigManager;
import net.runelite.client.eventbus.Subscribe;
import net.runelite.client.events.ConfigChanged;
import net.runelite.client.plugins.PluginDependency;
import net.runelite.client.plugins.PluginDescriptor;
import net.runelite.client.plugins.iutils.game.iNPC;
import net.runelite.client.plugins.iutils.iUtils;
import org.pf4j.Extension;

import javax.inject.Inject;

@Extension
@PluginDependency(Util.class)
@PluginDependency(iUtils.class)
@PluginDescriptor(
	name = "Chaos Fighter",
	description = "Wham! Bam! Thank you Ma'am!",
	enabledByDefault = false
)
@Slf4j
public class Fighter extends CScript {
	@Inject
	@Getter
	private Config config;

	@Provides
	Config provideConfig(ConfigManager configManager) {
		return configManager.getConfig(Config.class);
	}

	@Getter
	private String[] targetNames;

	@Getter
	private int[] targetIds;

	@Getter
	@Setter
	private iNPC currentTarget;

	@Override
	protected void onStart() {
		super.onStart();

		parseTargets();

		Run runTask = injector.getInstance(Run.class);
		runTask.setInterval(70, 95);
		tasks.add(runTask);
	}

	@SuppressWarnings("unused")
	@Subscribe
	private void onConfigChanged(ConfigChanged event) {
		if (!event.getGroup().equals("chaosfighter")) return;

		if (event.getKey().equals("targetNames") || event.getKey().equals("targetIds")) {
			parseTargets();
		}
	}

	@SuppressWarnings("unused")
	@Subscribe
	private void onActorDeath(ActorDeath event) {
		if (!(event.getActor() instanceof  NPC)) return;

		NPC actor = (NPC) event.getActor();

		if (actor.getIndex() == currentTarget.index()) {
			log.info("Our target died");
			currentTarget = null;
		}
	}

	private void parseTargets() {
		if (!config.targetIds().equals("")) {
			String[] rawTargetIds = config.targetIds().split(";");
			targetIds = new int[rawTargetIds.length];
			int i = 0;

			for (String id : rawTargetIds) {
				try {
					targetIds[i++] = Integer.parseInt(id.trim());
				} catch (NumberFormatException e) {
					log.error("Error parsing target ID: " + id.trim());
					targetIds[i-1] = 0;
				}
			}
		} else {
			targetIds = new int[0];
		}

		if (!config.targetNames().equals("")) {
			String[] rawTargetNames = config.targetNames().split(";");
			targetNames = new String[rawTargetNames.length];
			int i = 0;

			for (String name : rawTargetNames) {
				targetNames[i++] = name.trim();
			}
		} else {
			targetNames = new String[0];
		}
	}
}