package io.reisub.openosrs.herblore;

import com.google.inject.Provides;
import io.reisub.openosrs.herblore.tasks.Clean;
import io.reisub.openosrs.herblore.tasks.HandleBank;
import io.reisub.openosrs.herblore.tasks.MakePotion;
import io.reisub.openosrs.herblore.tasks.MakeUnfinished;
import io.reisub.openosrs.util.CScript;
import io.reisub.openosrs.util.Util;
import io.reisub.openosrs.util.enums.Activity;
import lombok.extern.slf4j.Slf4j;
import net.runelite.api.InventoryID;
import net.runelite.api.events.ItemContainerChanged;
import net.runelite.client.config.ConfigManager;
import net.runelite.client.eventbus.Subscribe;
import net.runelite.client.plugins.PluginDependency;
import net.runelite.client.plugins.PluginDescriptor;
import net.runelite.client.plugins.iutils.iUtils;
import org.pf4j.Extension;

import javax.inject.Inject;

@Extension
@PluginDependency(Util.class)
@PluginDependency(iUtils.class)
@PluginDescriptor(
	name = "Chaos Herblore",
	description = "You put the lime in the coconut, you drank them both up",
	enabledByDefault = false
)
@Slf4j
public class Herblore extends CScript {
	@Inject
	private Config config;

	@Provides
	Config provideConfig(ConfigManager configManager) {
		return configManager.getConfig(Config.class);
	}

	@Override
	protected void onStart() {
		super.onStart();

		idleCheckInventoryChange = true;

		addTask(Clean.class);
		addTask(MakeUnfinished.class);
		addTask(MakePotion.class);
		addTask(HandleBank.class);
	}

	@SuppressWarnings("unused")
	@Subscribe
	private void onItemContainerChanged(ItemContainerChanged event) {
		if (!isLoggedIn() || event.getItemContainer() != game.client().getItemContainer(InventoryID.INVENTORY)) return;

		int grimyHerbs = (int) game.inventory().withId(getGrimyHerbIds()).count();
		int cleanHerbs = (int) game.inventory().withId(getCleanHerbIds()).count();
		int bases = (int) game.inventory().withId(getBaseIds()).count();

		if (grimyHerbs == 0 && currentActivity == Activity.CLEANING_HERBS) {
			setActivity(Activity.IDLE);
		} else if (cleanHerbs == 0 && currentActivity == Activity.CREATING_UNFINISHED_POTIONS) {
			setActivity(Activity.IDLE);
		} else if (bases == 0 && currentActivity == Activity.CREATING_POTIONS) {
			setActivity(Activity.IDLE);
		}
	}

	public boolean shouldClean() {
		HerbloreTask task = config.herbloreTask();

		return task == HerbloreTask.CLEAN_HERBS
				|| task == HerbloreTask.CLEAN_HERBS_AND_MAKE_UNFINISHED
				|| task == HerbloreTask.CLEAN_HERBS_AND_MAKE_UNFINISHED_AND_MAKE_POTION;
	}

	public boolean shouldMakeUnfinished() {
		HerbloreTask task = config.herbloreTask();

		return task == HerbloreTask.MAKE_UNFINISHED
				|| task == HerbloreTask.CLEAN_HERBS_AND_MAKE_UNFINISHED
				|| task == HerbloreTask.MAKE_UNFINISHED_AND_MAKE_POTION
				|| task == HerbloreTask.CLEAN_HERBS_AND_MAKE_UNFINISHED_AND_MAKE_POTION;
	}

	public boolean shouldMakePotion() {
		HerbloreTask task = config.herbloreTask();

		return task == HerbloreTask.MAKE_POTION
				|| task == HerbloreTask.MAKE_UNFINISHED_AND_MAKE_POTION
				|| task == HerbloreTask.CLEAN_HERBS_AND_MAKE_UNFINISHED_AND_MAKE_POTION;
	}

	public Herb getHerb() {
		Herb herb;

		if (shouldMakePotion()) {
			herb = config.makePotion().getHerb();
		} else {
			herb = config.herbType();
		}

		return herb;
	}

	public int[] getGrimyHerbIds() {
		Herb herb = getHerb();

		if (herb == null) {
			return new int[]{};
		} else if (herb == Herb.ALL) {
			return Herb.getAllGrimyIds();
		} else {
			return new int[]{ herb.getGrimyId() };
		}
	}

	public int[] getCleanHerbIds() {
		Herb herb = getHerb();

		if (herb == null) {
			return new int[]{};
		} else if (herb == Herb.ALL) {
			return Herb.getAllCleanIds();
		} else {
			return new int[]{ herb.getCleanId() };
		}
	}

	public int[] getBaseIds() {
		Potion potion = config.makePotion();

		if (potion.getHerb() == null || potion.getSecondaryId() == -1) {
			return new int[]{ potion.getPotionBaseId() };
		} else {
			return new int[]{ potion.getHerb().getUnfinishedId() };
		}
	}

	public int[] getSecondaryIds() {
		if (config.makePotion().getSecondaryId() == -1) {
			return new int[]{ config.makePotion().getHerb().getCleanId() };
		} else {
			return new int[]{ config.makePotion().getSecondaryId() };
		}
	}
}