package io.reisub.openosrs.herblore.tasks;

import io.reisub.openosrs.herblore.Config;
import io.reisub.openosrs.herblore.Herblore;
import io.reisub.openosrs.herblore.Potion;
import io.reisub.openosrs.util.Task;
import io.reisub.openosrs.util.enums.Activity;
import net.runelite.api.ItemID;
import net.runelite.client.plugins.iutils.game.iObject;

import javax.inject.Inject;
import java.time.Duration;
import java.time.Instant;

public class HandleBank extends Task {
    @Inject
    private Herblore plugin;

    @Inject
    private Config config;

    private Instant lastBanking = Instant.EPOCH;

    @Override
    public String getStatus() {
        return "Banking";
    }

    @Override
    public boolean validate() {
        return plugin.getCurrentActivity() == Activity.IDLE
                & Duration.between(lastBanking, Instant.now()).getSeconds() > 5;
    }

    @Override
    public void execute() {
        if (!bank.isOpen()) {
            iObject bankObj = game.objects().withName("Bank chest", "Bank booth", "Bank Chest-wreck").nearest();
            if (bankObj == null) return;

            if (bankObj.actions().contains("Bank")) {
                bankObj.interact("Bank");
            } else {
                bankObj.interact(0);
            }

            game.waitUntil(() -> bank.isOpen(), 15);
        }

        if (game.inventory().all().size() > 0) {
            bank.depositInventory();
            game.sleepDelay();
        }

        if (plugin.shouldClean() && plugin.shouldMakeUnfinished() && plugin.shouldMakePotion()) {
            int quantity = withdraw(new int[]{config.makePotion().getPotionBaseId()}, 9);
            if (quantity == 0) {
                plugin.stop("Out of potion bases, stopping plugin.");
            }

            quantity = withdraw(plugin.getGrimyHerbIds(), 9);

            if (quantity < 9) {
                quantity = withdraw(plugin.getCleanHerbIds(), 9);
            }

            if (quantity == 0) {
                plugin.stop("Out of herbs, stopping plugin.");
            }

            quantity = withdraw(plugin.getSecondaryIds(), 9);
            if (quantity == 0) {
                plugin.stop("Out of secondaries, stopping plugin.");
            }
        } else if (plugin.shouldClean() && plugin.shouldMakeUnfinished()) {
            if (bank.quantity(ItemID.VIAL_OF_WATER) == 0) {
                plugin.stop("Out of vials, stopping plugin.");
            }

            bank.withdraw(ItemID.VIAL_OF_WATER, 14, false);
            game.sleepDelay();

            int quantity = withdraw(plugin.getGrimyHerbIds(), 14);

            if (quantity < 14) {
                quantity = withdraw(plugin.getCleanHerbIds(), 14);
            }

            if (quantity == 0) {
                plugin.stop("Out of herbs, stopping plugin.");
            }
        } else if (plugin.shouldClean()) {
            int quantity = withdraw(plugin.getGrimyHerbIds(), 28);

            if (quantity == 0) {
                plugin.stop("Out of herbs, stopping plugin.");
            }
        } else if (plugin.shouldMakeUnfinished() && plugin.shouldMakePotion()) {
            int quantity = withdraw(new int[]{config.makePotion().getPotionBaseId()}, 9);
            if (quantity == 0) {
                plugin.stop("Out of potion bases, stopping plugin.");
            }

            quantity = withdraw(plugin.getCleanHerbIds(), 9);
            if (quantity == 0) {
                plugin.stop("Out of clean herbs, stopping plugin.");
            }

            quantity = withdraw(plugin.getSecondaryIds(), 9);
            if (quantity == 0) {
                plugin.stop("Out of secondaries, stopping plugin.");
            }
        } else if (plugin.shouldMakeUnfinished()) {
            if (bank.quantity(ItemID.VIAL_OF_WATER) == 0) {
                plugin.stop("Out of vials, stopping plugin.");
            }

            bank.withdraw(ItemID.VIAL_OF_WATER, 14, false);
            game.sleepDelay();

            int quantity = withdraw(plugin.getCleanHerbIds(), 14);

            if (quantity == 0) {
                plugin.stop("Out of clean herbs, stopping plugin.");
            }
        } else if (plugin.shouldMakePotion()) {
            int quantity = withdraw(plugin.getBaseIds(), 14);
            if (quantity == 0) {
                plugin.stop("Out of potion bases, stopping plugin.");
            }

            quantity = withdraw(plugin.getSecondaryIds(), 14);
            if (quantity == 0) {
                plugin.stop("Out of secondaries, stopping plugin.");
            }
        }

        bank.close();
        game.waitUntil(() -> !bank.isOpen(), 5);
        lastBanking = Instant.now();
    }

    private int withdraw(int[] ids, int quantity) {
        int available = 0;

        for (int id : ids) {
            available += bank.quantity(id);
            bank.withdraw(id, quantity, false);
            game.sleepDelay();

            if (available >= quantity) break;
        }

        return available;
    }
}
