package io.reisub.openosrs.miner;

import com.google.inject.Provides;
import io.reisub.openosrs.miner.tasks.*;
import io.reisub.openosrs.util.CScript;
import io.reisub.openosrs.util.Util;
import io.reisub.openosrs.util.tasks.Run;
import lombok.Getter;
import lombok.Setter;
import lombok.extern.slf4j.Slf4j;
import net.runelite.api.GameState;
import net.runelite.api.ItemID;
import net.runelite.api.events.ChatMessage;
import net.runelite.api.events.GameTick;
import net.runelite.api.events.ItemContainerChanged;
import net.runelite.client.config.ConfigManager;
import net.runelite.client.eventbus.Subscribe;
import net.runelite.client.plugins.PluginDependency;
import net.runelite.client.plugins.PluginDescriptor;
import net.runelite.client.plugins.iutils.iUtils;
import org.pf4j.Extension;

import javax.inject.Inject;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

@Extension
@PluginDependency(Util.class)
@PluginDependency(iUtils.class)
@PluginDescriptor(
	name = "Chaos Miner",
	description = "Mines stuff",
	enabledByDefault = false
)
@Slf4j
public class Miner extends CScript {
	@Inject
	private Config config;

	@Provides
    Config provideConfig(ConfigManager configManager) {
		return configManager.getConfig(Config.class);
	}

	@Getter
	@Setter
	private boolean startedCycle;

	@Getter
	@Setter
	private boolean interrupted;

	private int ticks;

	@Getter
	private int ticksWithoutSuccess;

	private Mine mineTask;
	private ScheduledExecutorService executor;

	@Override
	protected void onStart() {
		super.onStart();

		Run runTask = injector.getInstance(Run.class);
		runTask.setInterval(70, 95);

		ticks = 0;

		tasks.add(runTask);
		addTask(TpAway.class);
		addTask(PrepareMining.class);
		addTask(PrepareBirdhouseRun.class);
		//addTask(Drop.class);
		if (!config.threeTickFourSandstone() && !config.threeTickFourGranite()) {
			addTask(Mine.class);
		} else {
			mineTask = injector.getInstance(Mine.class);
			executor = Executors.newSingleThreadScheduledExecutor();
			addTask(WalkToStartPosition.class);
		}
		addTask(Deposit.class);
	}

	@Override
	protected void onStop() {
		super.onStop();
		mineTask = null;
		executor = null;
	}

	public boolean hasWaterSkins() {
		return game.inventory().withId(
				ItemID.WATERSKIN1,
				ItemID.WATERSKIN2,
				ItemID.WATERSKIN3,
				ItemID.WATERSKIN4
		).exists();
	}

	@SuppressWarnings("unused")
	@Subscribe
	private void onGameTick(GameTick event) {
		if (mineTask != null && executor != null) {
			if (ticksWithoutSuccess++ > 15) {
				ticks = -6;
				ticksWithoutSuccess = 0;
				return;
			}

			executor.schedule(() -> {
				ticks++;
				if (ticks == 3) ticks = 0;

				if (ticks == 0 && mineTask.validate()) {
					mineTask.execute();
				}
			}, calc.random(245, 250), TimeUnit.MILLISECONDS);
			//}, calc.random(245, 250), TimeUnit.MILLISECONDS);
		}
	}

	@SuppressWarnings("unused")
	@Subscribe
	private void onItemContainerChanged(ItemContainerChanged event) {
		if (game.client().getGameState() != GameState.LOGGED_IN) return;

		game.inventory().withNamePart("Granite").drop();
		if (game.inventory().withId(ItemID.WATERSKIN0).exists()) {
			game.inventory().withId(ItemID.WATERSKIN0).drop();
			interrupted = true;
		}
	}

	@SuppressWarnings("unused")
	@Subscribe
	private void onChatMessage(ChatMessage event) {
		if (event.getMessage().equals("You take a drink of water.")) {
			interrupted = true;
		}

		if (event.getMessage().startsWith("You manage to quarry")) {
			ticksWithoutSuccess = 0;
		}
	}
}