package io.reisub.openosrs.miner.tasks;

import io.reisub.openosrs.miner.Config;
import io.reisub.openosrs.miner.Miner;
import io.reisub.openosrs.miner.RockPosition;
import io.reisub.openosrs.util.Task;
import net.runelite.api.ItemID;
import net.runelite.client.plugins.iutils.game.InventoryItem;
import net.runelite.client.plugins.iutils.game.iObject;
import net.runelite.client.plugins.iutils.scene.Position;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Queue;

public class Mine extends Task {
    @Inject
    private Miner plugin;

    @Inject
    private Config config;

    private final List<Position> positions = new ArrayList<>() {
        {
            add(new Position(3167, 2913, 0));
            add(new Position(3166, 2913, 0));
            add(new Position(3167, 2915, 0));
            add(new Position(3166, 2915, 0));
        }
    };

    private final Queue<Position> threeTickGranitePositions = new LinkedList<>() {
        {
            add(new Position(3165, 2908, 0));
            add(new Position(3165, 2909, 0));
            add(new Position(3165, 2910, 0));
            add(new Position(3167, 2911, 0));
        }
    };

    private final Position sandstoneStartPosition = new Position(3165, 2914, 0);
    private final Queue<RockPosition> threeTickSandstonePositions = new LinkedList<>() {
        {
            add(new RockPosition(new Position(3166, 2913, 0), new Position(3166, 2914, 0)));
            add(new RockPosition(new Position(3164, 2915, 0), new Position(3165, 2915, 0)));
            add(new RockPosition(new Position(3164, 2914, 0), new Position(3165, 2914, 0)));
            add(new RockPosition(new Position(3167, 2913, 0), new Position(3166, 2914, 0)));
        }
    };

    private RockPosition currentRockPosition;

    @Override
    public String getStatus() {
        return "Mining";
    }

    @Override
    public boolean validate() {
        if (isThreeTick()
                && !plugin.isStartedCycle()
                && !game.localPlayer().position().equals(sandstoneStartPosition)) {
            return false;
        }

        return !game.inventory().full()
                && plugin.hasWaterSkins()
                && game.localPlayer().position().regionID() == 12589
                && (game.localPlayer().isIdle() || isThreeTick());
    }

    @Override
    public void execute() {
        iObject rock = getRock();

        if (isThreeTick() && !walking.isRunning()) {
            walking.setRun(true);
        }

        if (isThreeTick() && plugin.isStartedCycle()) {
            InventoryItem knife = game.inventory().withId(ItemID.KNIFE).first();
            InventoryItem logs = game.inventory().withId(ItemID.TEAK_LOGS, ItemID.MAHOGANY_LOGS).first();
            if (knife == null || logs == null) return;

            knife.useOn(logs);
            if (rock == null) {
                game.walkUtils.sceneWalk(currentRockPosition.getInteractFrom(), 0, 0);
            }
        }

        while (rock == null && isThreeTick()) {
            rock = game.objects().withPosition(currentRockPosition.getRock()).withId(11386, 11387).first();
        }

        if (rock == null || game.inventory().full()) return;
        rock.interact(0);

        if (!isThreeTick()) {
            iObject finalRock = rock;
            game.waitUntil(() -> {
                iObject emptyRock = game.objects().withPosition(finalRock.position()).withId(11390, 11391).first();
                return emptyRock != null;
            }, 10);
        }
    }

    private iObject getRock() {
        if (isThreeTick()) {
            if (!plugin.isStartedCycle()) {
                plugin.setStartedCycle(true);
                resetQueue();
            }

            Position position;

            if (config.threeTickFourSandstone()) {
                currentRockPosition = threeTickSandstonePositions.poll();
                threeTickSandstonePositions.add(currentRockPosition);
                position = currentRockPosition.getRock();
            } else {
                position = threeTickGranitePositions.poll();
                threeTickGranitePositions.add(position);
            }

            return game.objects().withPosition(position).withId(11386, 11387).first();
        } else {
            List<iObject> rocks = game.objects().withId(11386, 11387).filter(o -> {
                for (Position p : positions) {
                    if (p.equals(o.position())) return true;
                }

                return false;
            }).all();

            if (rocks == null || rocks.isEmpty()) return null;

            int x = game.localPlayer().position().x;

            for (iObject r : rocks) {
                if (r.position().x == x) {
                    return r;
                }
            }

            return rocks.get(calc.random(0, rocks.size()));
        }
    }

    private boolean isThreeTick() {
        return config.threeTickFourGranite() || config.threeTickFourSandstone();
    }

    private void resetQueue() {
        if (config.threeTickFourSandstone()) {
            resetThreeTickSandstone();
        } else {
            resetThreeTickGranite();
        }
    }

    private void resetThreeTickGranite() {
        threeTickGranitePositions.clear();
        threeTickGranitePositions.add(new Position(3165, 2908, 0));
        threeTickGranitePositions.add(new Position(3165, 2909, 0));
        threeTickGranitePositions.add(new Position(3165, 2910, 0));
        threeTickGranitePositions.add(new Position(3167, 2911, 0));
    }

    private void resetThreeTickSandstone() {
        threeTickSandstonePositions.clear();
        threeTickSandstonePositions.add(new RockPosition(new Position(3166, 2913, 0), new Position(3166, 2914, 0)));
        threeTickSandstonePositions.add(new RockPosition(new Position(3164, 2915, 0), new Position(3165, 2915, 0)));
        threeTickSandstonePositions.add(new RockPosition(new Position(3164, 2914, 0), new Position(3165, 2914, 0)));
        threeTickSandstonePositions.add(new RockPosition(new Position(3167, 2913, 0), new Position(3166, 2914, 0)));
    }
}
