package io.reisub.openosrs.miner.tasks;

import io.reisub.openosrs.miner.Config;
import io.reisub.openosrs.miner.Miner;
import io.reisub.openosrs.util.Task;
import net.runelite.api.ItemID;
import net.runelite.client.plugins.iutils.game.ItemQuantity;
import net.runelite.client.plugins.iutils.game.iObject;
import net.runelite.client.plugins.iutils.ui.Chatbox;

import javax.inject.Inject;
import java.time.Duration;
import java.time.Instant;
import java.util.Set;

public class PrepareBirdhouseRun extends Task {
    @Inject
    private Miner plugin;

    @Inject
    private Config config;

    private Instant lastBank = Instant.EPOCH;

    private final Set<Integer> PENDANTS = Set.of(
            ItemID.DIGSITE_PENDANT_1,
            ItemID.DIGSITE_PENDANT_2,
            ItemID.DIGSITE_PENDANT_3,
            ItemID.DIGSITE_PENDANT_4,
            ItemID.DIGSITE_PENDANT_5
    );

    private final Set<Integer> BIRDHOUSE_SEEDS = Set.of(
            ItemID.BARLEY_SEED,
            ItemID.HAMMERSTONE_SEED,
            ItemID.ASGARNIAN_SEED,
            ItemID.JUTE_SEED,
            ItemID.YANILLIAN_SEED,
            ItemID.KRANDORIAN_SEED
    );

    private final Set<Integer> RINGS = Set.of(
            ItemID.RING_OF_DUELING1,
            ItemID.RING_OF_DUELING2,
            ItemID.RING_OF_DUELING3,
            ItemID.RING_OF_DUELING4,
            ItemID.RING_OF_DUELING5,
            ItemID.RING_OF_DUELING6,
            ItemID.RING_OF_DUELING7,
            ItemID.RING_OF_DUELING8
    );

    @Override
    public String getStatus() {
        return "Preparing for birdhouse run";
    }

    @Override
    public boolean validate() {
        return Duration.between(lastBank, Instant.now()).getSeconds() > 5
                && game.localPlayer().position().regionID() == 9776
                && (
                        !game.inventory().withId(PENDANTS).exists()
                        || !game.inventory().withId(ItemID.CHISEL).exists()
                        || !game.inventory().withId(ItemID.HAMMER).exists()
                        || game.inventory().withId(config.birdhouseLogs().getId()).count() < 4
                        || game.inventory().withId(BIRDHOUSE_SEEDS).quantity() < 40
                        || (!game.inventory().withId(RINGS).exists() && !game.equipment().withId(RINGS).exists())
                );
    }

    @Override
    public void execute() {
        if (!bank.isOpen()) {
            iObject bankObj = game.objects().withName("Bank chest", "Bank booth", "Bank Chest-wreck").nearest();
            if (bankObj == null) return;

            bankObj.interact(0);
            game.waitUntil(() -> bank.isOpen(), 15);
        }

        if (!game.inventory().all().isEmpty()) {
            bank.depositInventory();
            game.sleepDelay();
        }

        withdrawPendant();
        withdrawRing();
        withdrawTools();
        withdrawLogs();
        withdrawSeeds();

        bank.close();
        game.sleepDelay();

        game.inventory().withId(RINGS).findFirst().ifPresent((ring) -> {
            game.waitUntil(() -> !bank.isOpen(), 5);
            ring.interact(0);
            game.waitUntil(() -> !game.inventory().withId(RINGS).exists(), 5);
        });

        teleportToFossilIsland();

        lastBank = Instant.now();
    }

    private void withdrawPendant() {
        if (!game.inventory().withId(PENDANTS).exists()) {

            for (Integer id : PENDANTS) {
                if (bank.contains(id)) {
                    bank.withdraw(id, 1, false);
                    game.sleepDelay();
                    break;
                }
            }
        }
    }

    private void withdrawRing() {
        if (!game.equipment().withId(RINGS).exists() && !game.inventory().withId(RINGS).exists()) {
            for (Integer id : RINGS) {
                if (bank.contains(id)) {
                    bank.withdraw(id, 1, false);
                    game.sleepDelay();
                    break;
                }
            }
        }
    }

    private void withdrawTools() {
        if (!game.inventory().withId(ItemID.CHISEL).exists()) {
            bank.withdraw(ItemID.CHISEL, 1, false);
            game.sleepDelay();
        }

        if (!game.inventory().withId(ItemID.HAMMER).exists()) {
            bank.withdraw(ItemID.HAMMER, 1, false);
            game.sleepDelay();
        }
    }

    private void withdrawLogs() {
        int requiredLogs = 4 - (int) game.inventory().withId(config.birdhouseLogs().getId()).count();

        if (requiredLogs > 0) {
            bank.withdraw(config.birdhouseLogs().getId(), requiredLogs, false);
        }

        game.sleepDelay();
    }

    private void withdrawSeeds() {
        if (game.inventory().withId(BIRDHOUSE_SEEDS).quantity() < 40) {
            for (Integer id : BIRDHOUSE_SEEDS) {
                if (bank.contains(new ItemQuantity(id, 40))) {
                    bank.withdraw(id, 40, false);
                    game.sleepDelay();
                    break;
                }
            }
        }
    }

    private void teleportToFossilIsland() {
        if (!game.waitUntil(() -> (
                game.inventory().withId(PENDANTS).exists()
                        && game.inventory().withId(ItemID.CHISEL).exists()
                        && game.inventory().withId(ItemID.HAMMER).exists()
                        && game.inventory().withId(config.birdhouseLogs().getId()).count() >= 4
                        && game.inventory().withId(BIRDHOUSE_SEEDS).quantity() >= 40
                        && (game.inventory().withId(RINGS).exists() || game.equipment().withId(RINGS).exists())
        ), 5)) return;

        game.inventory().withId(PENDANTS).findFirst().ifPresent((pendant) -> {
            pendant.interact("Rub");
            game.tick();
        });

        game.waitUntil(() -> chatbox.chatState() == Chatbox.ChatState.OPTIONS_CHAT, 5);

        chatbox.chooseOption(2);
        game.waitUntil(() -> game.localPlayer().position().regionID() == 14908, 15);
        game.tick();
    }
}
