package io.reisub.openosrs.mtahelper;

import com.google.inject.Provides;

import io.reisub.openosrs.mtahelper.tasks.*;
import io.reisub.openosrs.util.CScript;
import io.reisub.openosrs.util.Util;
import io.reisub.openosrs.util.tasks.Run;
import lombok.Getter;
import lombok.Setter;
import lombok.extern.slf4j.Slf4j;
import net.runelite.api.*;
import net.runelite.api.coords.Direction;
import net.runelite.api.coords.LocalPoint;
import net.runelite.api.coords.WorldPoint;
import net.runelite.api.events.*;
import net.runelite.api.widgets.WidgetID;
import net.runelite.client.config.ConfigManager;
import net.runelite.client.eventbus.Subscribe;
import net.runelite.client.input.KeyListener;
import net.runelite.client.input.KeyManager;
import net.runelite.client.plugins.PluginDependency;
import net.runelite.client.plugins.PluginDescriptor;
import net.runelite.client.plugins.iutils.iUtils;
import net.runelite.client.plugins.mta.telekinetic.Maze;
import org.pf4j.Extension;

import javax.inject.Inject;
import java.awt.*;
import java.awt.event.KeyEvent;
import java.time.Duration;
import java.time.Instant;
import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

@Extension
@PluginDependency(Util.class)
@PluginDependency(iUtils.class)
@PluginDescriptor(
	name = "Chaos MTA Helper",
	description = "Makes Mage Training Arena slightly less terrible.",
	enabledByDefault = false
)
@Slf4j
public class MTAHelper extends CScript implements KeyListener {
	@Inject
	private Config config;

	@Inject
	private KeyManager keyManager;

	private Instant lastAction = Instant.EPOCH;

	@Provides
	Config provideConfig(ConfigManager configManager) {
		return configManager.getConfig(Config.class);
	}

	public static final int MTA_REGION = 13462;

	@Getter
	@Setter
	private boolean castTelekineticGrab;

	@Getter
	@Setter
	private boolean castBonesToBananas;

	@Getter
	@Setter
	private long lastHighAlchemy;

	@Override
	protected void onStart() {
		super.onStart();

		keyManager.registerKeyListener(this);

		Run runTask = injector.getInstance(Run.class);
		runTask.setInterval(70, 95);
		tasks.add(runTask);

		addTask(CastTelekineticGrab.class);
		addTask(CastEnchant.class);
		addTask(CastHighAlch.class);
		addTask(EatPeach.class);
		addTask(CastBonesToBananas.class);
	}

	@Override
	protected void onStop() {
		super.onStop();
		keyManager.unregisterKeyListener(this);
	}

	@SuppressWarnings("unused")
	@Subscribe
	private void onGameTick(GameTick event) {
		if (!isLoggedIn()) return;


	}

	@SuppressWarnings("unused")
	@Subscribe
	private void onItemContainerChanged(ItemContainerChanged event) {

	}

	@SuppressWarnings("unused")
	@Subscribe
	private void onStatChanged(StatChanged event) {
		if (event.getSkill() == Skill.MAGIC) {
			lastHighAlchemy = game.ticks();
		}
	}

	@Override
	public void keyTyped(KeyEvent e) {

	}

	@Override
	public void keyPressed(KeyEvent e) {
		if (Duration.between(lastAction, Instant.now()).getSeconds() < 2) return;
		lastAction = Instant.now();

		if (config.enableTelegrabHotkey()
				&& config.spellHotkey().matches(e)
				&& game.widget(WidgetID.MTA_TELEKINETIC_GROUP_ID, 0) != null) {
			castTelekineticGrab = true;
		} else if (config.enableBonesHotkey()
				&& config.spellHotkey().matches(e)
				&& game.widget(WidgetID.MTA_GRAVEYARD_GROUP_ID, 0) != null) {
			castBonesToBananas = true;
		}
	}

	@Override
	public void keyReleased(KeyEvent e) {

	}
}