package io.reisub.openosrs.shopper;

import com.google.inject.Provides;
import io.reisub.openosrs.shopper.tasks.*;
import io.reisub.openosrs.util.CScript;
import io.reisub.openosrs.util.Util;
import io.reisub.openosrs.util.tasks.Run;
import lombok.Getter;
import lombok.Setter;
import lombok.extern.slf4j.Slf4j;
import net.runelite.api.events.GameTick;
import net.runelite.client.config.ConfigManager;
import net.runelite.client.eventbus.Subscribe;
import net.runelite.client.plugins.PluginDependency;
import net.runelite.client.plugins.PluginDescriptor;
import net.runelite.client.plugins.iutils.iUtils;
import org.pf4j.Extension;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.List;

@Extension
@PluginDependency(Util.class)
@PluginDependency(iUtils.class)
@PluginDescriptor(
	name = "Chaos Shopper",
	description = "Hops worlds and buys stuff from NPCs",
	enabledByDefault = false
)
@Slf4j
public class Shopper extends CScript {
	@Inject
	private Config config;

	@Getter
	private List<Item> items;

	@Getter
	@Setter
	private boolean hop;

	private Hop hopTask;

	@Provides
	@SuppressWarnings("unused")
    Config provideConfig(ConfigManager configManager) {
		return configManager.getConfig(Config.class);
	}

	@Override
	protected void onStart() {
		super.onStart();

		loadItems();

		Run runTask = injector.getInstance(Run.class);
		runTask.setInterval(70, 95);

		hopTask = injector.getInstance(Hop.class);

		tasks.add(hopTask);
		tasks.add(runTask);
		addTask(Buy.class);
		addTask(HandleBank.class);
		addTask(CloseShop.class);
		addTask(OpenShop.class);
	}

	@Subscribe
	private void onGameTick(GameTick event) {
		if (!isLoggedIn()) return;

		if (hopTask != null) hopTask.onGameTick();
	}

	private void loadItems() {
		items = new ArrayList<>();

		if (config.itemOneEnabled()) {
			items.add(new Item(config.itemOneId(), config.itemOneAmount(), config.itemOneMinInStore(), 0));
		}

		if (config.itemTwoEnabled()) {
			items.add(new Item(config.itemTwoId(), config.itemTwoAmount(), config.itemTwoMinInStore(), 0));
		}

		if (config.itemThreeEnabled()) {
			items.add(new Item(config.itemThreeId(), config.itemThreeAmount(), config.itemThreeMinInStore(), 0));
		}

		if (config.itemFourEnabled()) {
			items.add(new Item(config.itemFourId(), config.itemFourAmount(), config.itemFourMinInStore(), 0));
		}

		if (config.itemFiveEnabled()) {
			items.add(new Item(config.itemFiveId(), config.itemFiveAmount(), config.itemFiveMinInStore(), 0));
		}
	}
}