package io.reisub.openosrs.shopper.tasks;

import io.reisub.openosrs.shopper.Item;
import io.reisub.openosrs.shopper.Shopper;
import io.reisub.openosrs.util.Task;
import net.runelite.api.ItemID;
import net.runelite.client.plugins.iutils.game.iObject;

import javax.inject.Inject;
import java.time.Duration;
import java.time.Instant;

public class HandleBank extends Task {
    @Inject
    private Shopper plugin;

    private Instant lastBanking = Instant.EPOCH;

    @Override
    public String getStatus() {
        return "Banking";
    }

    @Override
    public boolean validate() {
        return game.inventory().full()
                && Duration.between(lastBanking, Instant.now()).getSeconds() > 3;
    }

    @Override
    public void execute() {
        if (!bank.isOpen()) {
            iObject bankObj = game.objects().withName("Bank chest", "Bank booth", "Bank Chest-wreck", "Bank chest").nearest();
            if (bankObj == null) return;

            if (bankObj.actions().contains("Bank")) {
                bankObj.interact("Bank");
            } else if (bankObj.actions().contains("Use")) {
                bankObj.interact("Use");
            } else {
                bankObj.interact(0);
            }

            game.waitUntil(() -> bank.isOpen(), 15);
        }

        for (Item item : plugin.getItems()) {
            int quantity = bank.quantity(item.getId()) + (int) game.inventory().withId(item.getId()).count();
            item.setAmountBought(quantity);

            bank.depositExcept(false, ItemID.COINS, ItemID.COINS_995, ItemID.COINS_6964, ItemID.COINS_8890);
            game.sleepDelay();
        }

        bank.close();
        game.sleepDelay();
        lastBanking = Instant.now();
    }
}
