package io.reisub.openosrs.shopper.tasks;

import io.reisub.openosrs.shopper.Config;
import io.reisub.openosrs.shopper.Shopper;
import io.reisub.openosrs.util.Task;
import net.runelite.api.GameState;
import net.runelite.api.widgets.WidgetInfo;
import net.runelite.client.game.WorldService;
import net.runelite.client.util.WorldUtil;
import net.runelite.http.api.worlds.World;
import net.runelite.http.api.worlds.WorldType;

import javax.inject.Inject;
import java.time.Duration;
import java.time.Instant;
import java.util.LinkedList;
import java.util.Queue;

public class Hop extends Task {
    @Inject
    private Shopper plugin;

    @Inject
    public WorldService worldService;

    private Queue<World> worldQueue;
    private net.runelite.api.World quickHopTargetWorld;
    private int displaySwitcherAttempts = 0;
    private Instant lastAttempt = Instant.EPOCH;

    @Override
    public String getStatus() {
        return "Hopping to next world";
    }

    @Override
    public boolean validate() {
        return plugin.isHop() && Duration.between(lastAttempt, Instant.now()).getSeconds() > 5;
    }

    @Override
    public void execute() {
        if (worldQueue == null) initializeWorldQueue();

        World world = worldQueue.poll();
        if (world == null) return;

        worldQueue.add(world);

        final net.runelite.api.World rsWorld = game.client().createWorld();
        rsWorld.setActivity(world.getActivity());
        rsWorld.setAddress(world.getAddress());
        rsWorld.setId(world.getId());
        rsWorld.setPlayerCount(world.getPlayers());
        rsWorld.setLocation(world.getLocation());
        rsWorld.setTypes(WorldUtil.toWorldTypes(world.getTypes()));

        if (game.client().getGameState() == GameState.LOGIN_SCREEN) {
            game.client().changeWorld(rsWorld);
            return;
        }

        quickHopTargetWorld = rsWorld;
        displaySwitcherAttempts = 0;
        lastAttempt = Instant.now();
        plugin.setHop(false);
        game.tick(3);
    }

    public void onGameTick() {
        if (quickHopTargetWorld == null) return;

        if (game.client().getWidget(WidgetInfo.WORLD_SWITCHER_LIST) == null) {
            game.client().openWorldHopper();
            if (++displaySwitcherAttempts >= 5) {
                logWarn("Failed to hop after 5 attempts");
                displaySwitcherAttempts = 0;
                quickHopTargetWorld = null;
            }
        } else {
            game.client().hopToWorld(quickHopTargetWorld);
            plugin.setLastHop(Instant.now());
            quickHopTargetWorld = null;
        }
    }

    private void initializeWorldQueue() {
        if (worldService.getWorlds() == null) return;

        int current = game.client().getWorld();
        if (current == 0 || current == -1) return;

        worldQueue = new LinkedList<>();

        for (World world : worldService.getWorlds().getWorlds()) {
            if (!world.getTypes().contains(WorldType.MEMBERS)) continue;

            if (world.getTypes().contains(WorldType.BOUNTY)
                    || world.getTypes().contains(WorldType.DEADMAN)
                    || world.getTypes().contains(WorldType.HIGH_RISK)
                    || world.getTypes().contains(WorldType.LAST_MAN_STANDING)
                    || world.getTypes().contains(WorldType.NOSAVE_MODE)
                    || world.getTypes().contains(WorldType.PVP)
                    || world.getTypes().contains(WorldType.SEASONAL)
                    || world.getTypes().contains(WorldType.TOURNAMENT)) {
                continue;
            }

            if (world.getTypes().contains(WorldType.SKILL_TOTAL)) {
                try {
                    int totalRequirement = Integer.parseInt(world.getActivity().substring(0, world.getActivity().indexOf(" ")));

                    if (game.client().getTotalLevel() < totalRequirement) {
                        continue;
                    }
                } catch (NumberFormatException e) {
                    logWarn("Failed to parse total level requirement for world " + world.getId());
                    continue;
                }
            }

            worldQueue.add(world);
        }

        while (true) {
            World world = worldQueue.poll();
            worldQueue.add(world);

            if (world == null || world.getId() == current) break;
        }
    }
}
