package io.reisub.openosrs.smelter.tasks;

import io.reisub.openosrs.smelter.Config;
import io.reisub.openosrs.smelter.Ingredient;
import io.reisub.openosrs.smelter.Product;
import io.reisub.openosrs.util.Task;
import net.runelite.api.ItemID;
import net.runelite.client.plugins.iutils.game.iObject;

import javax.inject.Inject;
import java.time.Duration;
import java.time.Instant;

public class HandleBank extends Task {
    @Inject
    private Config config;

    private Instant lastBanking = Instant.EPOCH;

    @Override
    public String getStatus() {
        return "Banking";
    }

    @Override
    public boolean validate() {
        boolean hasAllIngredients = true;

        for (Ingredient ingredient : config.targetProduct().getIngredients()) {
            if (!game.inventory().withId(ingredient.getId()).exists()) {
                hasAllIngredients = false;
            }
        }

        return !hasAllIngredients
                && game.localPlayer().isIdle()
                && Duration.between(lastBanking, Instant.now()).getSeconds() > 5;
    }

    @Override
    public void execute() {
        if (!bank.isOpen()) {
            iObject bankObj = game.objects().withName("Bank booth").nearest();
            if (bankObj == null) return;

            bankObj.interact("Bank");
            game.waitUntil(() -> bank.isOpen(), 15);
        }

        if (config.targetProduct() == Product.CANNONBALLS) {
            bank.depositExcept(false, ItemID.AMMO_MOULD);
        } else {
            bank.depositInventory();
        }
        game.tick();
        game.sleepDelay();

        for (Ingredient ingredient : config.targetProduct().getIngredients()) {
            bank.withdraw(ingredient.getId(), ingredient.getAmount(), false);
            game.sleepDelay();
        }

        bank.close();
        game.sleepDelay();
        lastBanking = Instant.now();
    }
}
