package io.reisub.openosrs.smither.tasks;

import io.reisub.openosrs.smither.Config;
import io.reisub.openosrs.smither.Smither;
import io.reisub.openosrs.util.Task;
import io.reisub.openosrs.util.enums.Activity;
import net.runelite.api.ItemID;
import net.runelite.client.plugins.iutils.game.iObject;

import javax.inject.Inject;
import java.time.Duration;
import java.time.Instant;

public class HandleBank extends Task {
    @Inject
    private Smither plugin;

    @Inject
    private Config config;

    private Instant lastBanking = Instant.EPOCH;

    @Override
    public String getStatus() {
        return "Banking";
    }

    @Override
    public boolean validate() {
        return plugin.getCurrentActivity() == Activity.IDLE
                && game.inventory().withId(config.metalType().getBarId()).count() < config.product().getRequiredBars()
                && Duration.between(lastBanking, Instant.now()).getSeconds() > 5;
    }

    @Override
    public void execute() {
        if (!bank.isOpen()) {
            iObject bankObj = game.objects().withName("Bank chest", "Bank booth", "Bank Chest-wreck", "Bank chest").nearest();
            if (bankObj == null) return;

            if (bankObj.actions().contains("Bank")) {
                bankObj.interact("Bank");
            } else if (bankObj.actions().contains("Use")) {
                bankObj.interact("Use");
            } else {
                bankObj.interact(0);
            }

            game.waitUntil(() -> bank.isOpen(), 15);
        }

        int barCount = bank.quantity(config.metalType().getBarId()) + (int) game.inventory().withId(config.metalType().getBarId()).count();

        if (barCount < config.product().getRequiredBars()) {
            plugin.stop("Out of bars, stopping plugin.");
        }

        bank.depositExcept(false, ItemID.HAMMER, ItemID.IMCANDO_HAMMER, config.metalType().getBarId());
        game.sleepDelay();

        bank.withdraw(config.metalType().getBarId(), 28, false);
        game.sleepDelay();

        bank.close();
        game.sleepDelay();

        lastBanking = Instant.now();
    }
}
