package io.reisub.openosrs.tempoross.tasks;

import io.reisub.openosrs.tempoross.Activity;
import io.reisub.openosrs.tempoross.Tempoross;
import io.reisub.openosrs.util.Task;
import net.runelite.api.ItemID;
import net.runelite.api.NpcID;
import net.runelite.client.plugins.iutils.game.iNPC;
import net.runelite.client.plugins.iutils.game.iObject;
import net.runelite.client.plugins.iutils.scene.Position;

import javax.inject.Inject;

public class Fish extends Task {
    @Inject
    private Tempoross plugin;

    private final int normalSpotID = NpcID.FISHING_SPOT_10565;
    private final int doubleSpotID = NpcID.FISHING_SPOT_10569;

    @Override
    public String getStatus() {
        return "Fishing";
    }

    @Override
    public boolean validate() {
        if (!plugin.isInTemperossArea() || game.inventory().full()) return false;

        iNPC normalSpot = game.npcs().withId(normalSpotID).nearest();
        iNPC doubleSpot = game.npcs().withId(doubleSpotID).nearest();

        // interrupt cooking when double spot exists
        if (plugin.getCurrentActivity() == Activity.COOKING
                && doubleSpot != null) {
            return true;
        }

        // change from normal spot to double spot
        Position playerPos = game.localPlayer().position();
        if (plugin.getCurrentActivity() == Activity.FISHING
                && doubleSpot != null
                && doubleSpot.position().distanceTo(playerPos) > normalSpot.position().distanceTo(playerPos)) {
            return true;
        }

        // fish when idle and low on raw harpoonfish
        if (plugin.getCurrentActivity() == Activity.IDLE
                && game.inventory().withId(ItemID.RAW_HARPOONFISH).count() <= 8) {
            return true;
        }

        return false;
    }

    @Override
    public void execute() {
        iObject spot = game.objects().withId(doubleSpotID).nearest();

        if (spot == null) {
            spot = game.objects().withId(normalSpotID).nearest();
        }

        if (spot == null) return;

        spot.interact(0);
        game.waitUntil(() -> plugin.getCurrentActivity() == Activity.FISHING);
    }
}
