package io.reisub.openosrs.test;

import com.google.inject.Provides;
import io.reisub.openosrs.test.tasks.Test;
import io.reisub.openosrs.util.Task;
import io.reisub.openosrs.util.Util;
import lombok.extern.slf4j.Slf4j;
import net.runelite.api.Actor;
import net.runelite.api.MenuAction;
import net.runelite.api.MenuEntry;
import net.runelite.api.Prayer;
import net.runelite.api.events.*;
import net.runelite.api.widgets.Widget;
import net.runelite.api.widgets.WidgetInfo;
import net.runelite.api.widgets.WidgetItem;
import net.runelite.client.config.ConfigManager;
import net.runelite.client.eventbus.Subscribe;
import net.runelite.client.input.KeyListener;
import net.runelite.client.input.KeyManager;
import net.runelite.client.plugins.PluginDependency;
import net.runelite.client.plugins.PluginDescriptor;
import net.runelite.client.plugins.iutils.game.InventoryItem;
import net.runelite.client.plugins.iutils.iUtils;
import net.runelite.client.plugins.iutils.scripts.iScript;
import org.apache.commons.lang3.tuple.Pair;
import org.pf4j.Extension;

import javax.inject.Inject;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Locale;

@Extension
@PluginDependency(Util.class)
@PluginDependency(iUtils.class)
@PluginDescriptor(
	name = "Chaos Test",
	description = "Plugin for testing",
	enabledByDefault = false
)
@Slf4j
public class TestPlugin extends iScript implements KeyListener {
	@Inject
	private TestConfig config;

	@Inject
	private KeyManager keyManager;

	private List<Task> tasks;

	@Provides
	TestConfig provideConfig(ConfigManager configManager) {
		return configManager.getConfig(TestConfig.class);
	}

	@Override
	protected void loop() {
		for (Task t : tasks) {
			if (t.validate()) {
				log.info(t.getStatus());
				t.execute();
				break;
			}
		}

		game.sleepDelay();
	}

	@Override
	protected void onStart() {
		log.info("Starting Chaos Test");

		keyManager.registerKeyListener(this);

		tasks = new ArrayList<>();
		tasks.add(injector.getInstance(Test.class));
	}

	@Override
	protected void onStop() {
		log.info("Stopping Chaos Test");
		if (tasks != null) {
			tasks.clear();
		}

		keyManager.unregisterKeyListener(this);
	}

	@Subscribe
	private void onGameTick(GameTick event) {

	}

	@Subscribe
	private void onConfigButtonPressed(ConfigButtonClicked configButtonClicked) {
		if (configButtonClicked.getKey().equals("startButton")) {
			execute();
		}
	}

	@Subscribe
	private void onScriptPreFired(ScriptPreFired event) {
//		int id = event.getScriptId();
//		if (id > 1000 || id == 900 || id == 100 || id == 44) return;
//		if (event.getScriptEvent() == null) return;
//		log.info("id: " + event.getScriptId());
	}

	@Subscribe
	private void onAnimationChanged(AnimationChanged event) {
		Actor actor = event.getActor();
		if (actor == null || actor.getName() == null) return;

		if (actor.getName().equals(game.localPlayer().name())) {
			log.info("player anim: " + actor.getAnimation());
		}

		if (actor.getName().toLowerCase(Locale.ROOT).contains("hespori")) {
			log.info("hespori anim: " + actor.getAnimation());
		}

//		if (event.getActor() != null) {
//			log.info("actor: " + event.getActor().getName());
//			log.info("id: " + event.getActor().getAnimation());
//		}
	}

	@Subscribe
	private void onProjectileMoved(ProjectileMoved event) {
//		if (event.getProjectile().getInteracting() != null) {
//			log.info("projectile interacting: " + event.getProjectile().getInteracting().getName());
//			log.info("projectile id: " + event.getProjectile().getId());
//		}
	}

	@Subscribe
	private void onHitsplatApplied(HitsplatApplied event) {


	}

	@Override
	public void keyTyped(KeyEvent e) {

	}

	@Override
	public void keyPressed(KeyEvent e) {
		if (config.hotkeyOne().matches(e)) {
			log.info("hotkey 1 pressed: magic");
		} else if (config.hotkeyTwo().matches(e)) {
			log.info("hotkey 2 pressed: ranged");
		}
	}

	@Override
	public void keyReleased(KeyEvent e) {

	}
}