package io.reisub.openosrs.util.tasks;

import io.reisub.openosrs.util.Task;
import lombok.Getter;
import lombok.Setter;
import net.runelite.api.Skill;

public class Eat extends Task {
    private long last = 0;
    private int min = 5;
    private int max = 10;

    @Setter
    private boolean wait = true;

    @Getter
    private int threshold = 5;

    public void setInterval(int min, int max) {
        this.min = min;
        this.max = max;

        threshold = getNewThreshold();
    }

    @Override
    public String getStatus() {
        return "Eating";
    }

    @Override
    public boolean validate() {
        return game.modifiedLevel(Skill.HITPOINTS) <= threshold
                && game.inventory().withAction("Eat", "Drink").findAny().isPresent()
                && System.currentTimeMillis() > last + 1800;
    }

    @Override
    public void execute() {
        int hp = game.modifiedLevel(Skill.HITPOINTS);

        game.inventory().withAction("Eat", "Drink").findFirst().ifPresent(inventoryItem -> {
            if (inventoryItem.actions().contains("Eat")) {
                inventoryItem.interact("Eat");
            } else {
                inventoryItem.interact("Drink");
            }
            last = System.currentTimeMillis();
        });
        if (wait) game.tick();

        threshold = getNewThreshold();

        if (wait) game.waitUntil(() -> hp < game.modifiedLevel(Skill.HITPOINTS), 10);
    }

    private int getNewThreshold() {
        return calc.random(min, max + 1);
    }
}
