package io.reisub.openosrs.volcanicashminer;

import com.google.inject.Provides;
import io.reisub.openosrs.util.CScript;
import io.reisub.openosrs.util.Util;
import io.reisub.openosrs.util.enums.Activity;
import io.reisub.openosrs.util.tasks.Run;
import io.reisub.openosrs.volcanicashminer.tasks.Drop;
import io.reisub.openosrs.volcanicashminer.tasks.Mine;
import lombok.extern.slf4j.Slf4j;
import net.runelite.api.AnimationID;
import net.runelite.api.GameObject;
import net.runelite.api.GameState;
import net.runelite.api.events.AnimationChanged;
import net.runelite.api.events.GameObjectDespawned;
import net.runelite.api.events.ItemContainerChanged;
import net.runelite.client.config.ConfigManager;
import net.runelite.client.eventbus.Subscribe;
import net.runelite.client.plugins.PluginDependency;
import net.runelite.client.plugins.PluginDescriptor;
import net.runelite.client.plugins.iutils.iUtils;
import org.pf4j.Extension;

@Extension
@PluginDependency(Util.class)
@PluginDependency(iUtils.class)
@PluginDescriptor(
	name = "Chaos Volcanic Ash Miner",
	description = "Mines volcanic ash.",
	enabledByDefault = false
)
@Slf4j
public class VolcanicAshMiner extends CScript {
	@Provides
	Config provideConfig(ConfigManager configManager) {
		return configManager.getConfig(Config.class);
	}

	@Override
	protected void onStart() {
		super.onStart();

		Run runTask = injector.getInstance(Run.class);
		runTask.setInterval(70, 95);
		tasks.add(runTask);
		addTask(Drop.class);
		addTask(Mine.class);
	}

	@SuppressWarnings("unused")
	@Subscribe
	private void onAnimationChanged(AnimationChanged event) {
		if (game.client().getGameState() != GameState.LOGGED_IN) return;
		if (event.getActor() == null || event.getActor().getName() == null) return;

		if (!event.getActor().getName().equals(game.localPlayer().name())) return;

		switch (game.localPlayer().animation()) {
			case AnimationID.MINING_BRONZE_PICKAXE:
			case AnimationID.MINING_IRON_PICKAXE:
			case AnimationID.MINING_STEEL_PICKAXE:
			case AnimationID.MINING_BLACK_PICKAXE:
			case AnimationID.MINING_MITHRIL_PICKAXE:
			case AnimationID.MINING_ADAMANT_PICKAXE:
			case AnimationID.MINING_RUNE_PICKAXE:
			case AnimationID.MINING_GILDED_PICKAXE:
			case AnimationID.MINING_DRAGON_PICKAXE:
			case AnimationID.MINING_DRAGON_PICKAXE_UPGRADED:
			case AnimationID.MINING_INFERNAL_PICKAXE:
			case AnimationID.MINING_CRYSTAL_PICKAXE:
				setActivity(Activity.MINING);
				break;
		}
	}

	@SuppressWarnings("unused")
	@Subscribe
	private void onGameObjectDespawned(GameObjectDespawned event) {
		if (game.client().getGameState() != GameState.LOGGED_IN) return;
		GameObject object = event.getGameObject();

		if (object.getName().equals("Ash pile") && game.client().getLocalPlayer().getWorldLocation().distanceTo(object.getWorldLocation()) < 3) {
			setActivity(Activity.IDLE);
		}
	}

	@SuppressWarnings("unused")
	@Subscribe
	private void onItemContainerChanged(ItemContainerChanged event) {
		if (game.client().getGameState() != GameState.LOGGED_IN) return;

		if (game.inventory().withName("Soda ash").exists()) {
			setActivity(Activity.IDLE);
		}
	}
}