package io.reisub.openosrs.wintertodt;

import com.google.gson.*;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import net.runelite.client.plugins.iutils.game.Game;
import okhttp3.*;
import org.jetbrains.annotations.NotNull;

import javax.inject.Inject;
import javax.inject.Singleton;
import java.io.IOException;
import java.time.Duration;
import java.time.Instant;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@Slf4j
@Singleton
public class Scouter {
    @Inject
    public Wintertodt plugin;

    @Inject
    public Game game;

    @Inject
    public OkHttpClient okHttpClient;

    @Getter
    private final List<BossData> data = new ArrayList<>(4);

    private Instant lastUpdate;

    public void onGameTick() {
        if (lastUpdate == null || Duration.between(lastUpdate, Instant.now()).getSeconds() > 2) {
            lastUpdate = Instant.now();

            getRemoteData();
        }
    }

    public BossData getLocalData() {
        return new BossData(
                plugin.getBossHealth(),
                game.client().getWorld(),
                System.currentTimeMillis() / 1000,
                plugin.getRespawnTimer()
        );
    }

    private void getRemoteData() {
        try {
            Request r = new Request.Builder()
                    .url("https://www.wintertodt.com/scouter/")
                    .addHeader("User-Agent", "RuneLite")
                    .addHeader("Authorization", "2")
                    .build();

            okHttpClient.newCall(r).enqueue(new Callback() {
                @Override
                public void onFailure(@NotNull Call call, @NotNull IOException e) {
                    log.debug("Error retrieving Wintertodt boss data", e);
                }

                @Override
                public void onResponse(@NotNull Call call, @NotNull Response response) {
                    if (response.isSuccessful()) {
                       try {
                           JsonArray jsonArray = new Gson().fromJson(Objects.requireNonNull(response.body()).string(), JsonArray.class);
                           parseData(jsonArray);
                       } catch (IOException | JsonSyntaxException ignored) {}
                    }
                }
            });
        } catch (IllegalArgumentException ignored) {}
    }

    private void parseData(JsonArray jsonArray) {
        for (JsonElement jsonElement : jsonArray) {
            JsonObject jsonObject = jsonElement.getAsJsonObject();
            try {
                BossData bossData = new BossData(jsonObject.get("a").getAsInt(), jsonObject.get("b").getAsInt(), jsonObject.get("c").getAsLong(), jsonObject.get("d").getAsInt());
                if (bossData.getWorld() != game.client().getWorld()) {
                    data.add(bossData);
                }
            } catch (UnsupportedOperationException ignored) {}
        }
    }
}
