package io.reisub.openosrs.wintertodt.tasks;

import io.reisub.openosrs.util.Task;
import io.reisub.openosrs.wintertodt.BossData;
import io.reisub.openosrs.wintertodt.Config;
import io.reisub.openosrs.wintertodt.Scouter;
import io.reisub.openosrs.wintertodt.Wintertodt;
import net.runelite.api.GameState;
import net.runelite.api.widgets.WidgetInfo;
import net.runelite.client.game.WorldService;
import net.runelite.client.util.WorldUtil;
import net.runelite.http.api.worlds.World;
import net.runelite.http.api.worlds.WorldResult;

import javax.inject.Inject;
import java.time.Duration;
import java.time.Instant;
import java.util.List;

public class Hop extends Task {
    @Inject
    public Wintertodt plugin;

    @Inject
    public Config config;

    @Inject
    public Scouter scouter;

    @Inject
    public WorldService worldService;

    private Instant lastAttempt;

    @Override
    public String getStatus() {
        return "Hopping to other world";
    }

    private BossData best;
    private net.runelite.api.World quickHopTargetWorld;
    private int displaySwitcherAttempts = 0;

    @Override
    public boolean validate() {
        if (!plugin.isInWintertodtRegion()) return false;
        if (plugin.bossIsUp()) return false;

        best = findBestWorld();

        return best.getWorld() != game.client().getWorld()
                && (lastAttempt == null || Duration.between(lastAttempt, Instant.now()).getSeconds() >= 5);
    }

    @Override
    public void execute() {
        WorldResult worldResult = worldService.getWorlds();
        if (worldResult == null) return;

        World world = worldResult.findWorld(best.getWorld());
        if (world == null) return;

        final net.runelite.api.World rsWorld = game.client().createWorld();
        rsWorld.setActivity(world.getActivity());
        rsWorld.setAddress(world.getAddress());
        rsWorld.setId(world.getId());
        rsWorld.setPlayerCount(world.getPlayers());
        rsWorld.setLocation(world.getLocation());
        rsWorld.setTypes(WorldUtil.toWorldTypes(world.getTypes()));

        if (game.client().getGameState() == GameState.LOGIN_SCREEN) {
            game.client().changeWorld(rsWorld);
            return;
        }

        quickHopTargetWorld = rsWorld;
        displaySwitcherAttempts = 0;
        lastAttempt = Instant.now();
    }

    public void onGameTick() {
        if (quickHopTargetWorld == null) return;

        if (game.client().getWidget(WidgetInfo.WORLD_SWITCHER_LIST) == null) {
            game.client().openWorldHopper();
            if (++displaySwitcherAttempts >= 5) {
                logWarn("Failed to hop after 5 attempts");
                displaySwitcherAttempts = 0;
                quickHopTargetWorld = null;
            }
        } else {
            game.client().hopToWorld(quickHopTargetWorld);
            plugin.setLastHop(Instant.now());
            quickHopTargetWorld = null;
        }
    }

    private BossData findBestWorld() {
        List<BossData> dataList = scouter.getData();

        BossData best = scouter.getLocalData();

        for (BossData data : dataList) {
            // ignore data older than 5 seconds
            if (System.currentTimeMillis() - (data.getTime() * 1000) >= 5000) continue;

            if (best.getHealth() > 0) {
                if (data.getHealth() >= config.hopPercentage() && data.getHealth() < best.getHealth()) {
                    best = data;
                }
            } else {
                if (data.getHealth() >= config.hopPercentage()) {
                    best = data;
                } else if (data.getHealth() <= 0 && data.getTimer() < best.getTimer()) {
                    best = data;
                }
            }
        }

        return best;
    }
}
